/******************************************************************************

                           Copyright (c) 2014, 2015
                        Lantiq Beteiligungs-GmbH & Co.KG
                             http://www.lantiq.com

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

/**
   \file drv_vmmc_bbd.c
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "drv_vmmc_api.h"

#ifdef VMMC_FEAT_SLIC

#include "drv_vmmc_bbd.h"
#include "drv_sdd_api.h"

/* ============================= */
/* Local defines                 */
/* ============================= */

/* ============================= */
/* BBD tag definitions           */
/* ============================= */

/* ============================= */
/* Global Structures             */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Local function definition     */
/* ============================= */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Download a BBD file.

   \param  pLLDev       Pointer to the VMMC device structure.
   \param  pProc        Pointer to low-level device initialization structure.

   \return
   - VMMC_statusOk if successful
*/
IFX_int32_t VMMC_TAPI_LL_BBD_Dnld(IFX_TAPI_LL_DEV_t *pLLDev,
                                  IFX_void_t const *pProc)
{
   VMMC_DEVICE *pDev = (VMMC_DEVICE*)pLLDev;
   VMMC_IO_INIT IoInit;
   IFX_int32_t ret = VMMC_statusOk;

   if (pProc == IFX_NULL)
   {
      /* reset all init pointers and init flag */
      memset (&IoInit, 0, sizeof (IoInit));
      /* set additional default flags */
      IoInit.nFlags = FW_AUTODWLD;
   }
   else
   {
      /* The init struct is specific for the LL driver and only known here.
         Because HL does not know the struct it cannot copy it and so it is
         copied here in the LL driver. */
      VMMC_OS_CpyUsr2Kern (&IoInit, pProc, sizeof (IoInit));
   }

   /* BBD Download */
   if(IoInit.pBBDbuf != IFX_NULL)
   {
      bbd_format_t bbdDwld;

      bbdDwld.buf  = IoInit.pBBDbuf;
      bbdDwld.size = IoInit.bbd_size;
      ret = VMMC_BBD_Download((VMMC_CHANNEL *) pDev, &bbdDwld);
      if (ret != IFX_SUCCESS)
      {
         TRACE(VMMC,DBG_LEVEL_HIGH, ("INFO: BBD Buffer Download failed.\n"));
      }
      else
      {
         TRACE(VMMC,DBG_LEVEL_LOW, ("INFO: BBD Buffer provided successfully.\n"));
      }
   }
   else
   {
      TRACE(VMMC,DBG_LEVEL_HIGH, ("INFO: No BBD Buffer provided.\n"));
   }

   RETURN_DEVSTATUS (ret);
}


/**
   Does a BBD download.

   Handles the IOCTL FIO_VMMC_BBD_DOWNLOAD to download a BBD buffer on a
   given channel or a given device. For a device the download is done on
   each of the channels.

   \param  pCh          Pointer to VMMC channel or VMMC device structure.
   \param  pBBD         Pointer to user BBD buffer.

   \return
   IFX_SUCCESS or error code.
*/
IFX_int32_t VMMC_BBD_Download (VMMC_CHANNEL *pCh, bbd_format_t *pBBD)
{
   IFX_TAPI_LL_CH_t *pSDD;

   /* distinguish device / channel context */
   if (pCh->nChannel == 0)
   {
      pSDD = ((VMMC_DEVICE *)pCh)->pSDD;
   }
   else
   {
      pSDD = pCh->pSDD;
   }

   /* Redirect to SDD handler */
   return SDD_BBD_EXT_Download (pSDD, pBBD);
}

#endif /* VMMC_FEAT_SLIC */
