#ifndef _DRV_VMMC_LOADER_HANDLER_H
#define _DRV_VMMC_LOADER_HANDLER_H
/******************************************************************************

                            Copyright (c) 2014-2015
                        Lantiq Beteiligungs-GmbH & Co.KG
                             http://www.lantiq.com

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

 ******************************************************************************/

/**
   \file drv_vmmc_loader_handler.h
   This file contains the implementation vmmc loader routines.
*/

/**
   \mainpage VMMC loader
   The VMMC loader allows to run VOICE firmware on MIPS VPE using VMB interface.
*/

/* ========================================================================== */
/*                                 Includes                                   */
/* ========================================================================== */
#include "drv_config.h"
#include <linux/kernel.h>

/* ========================================================================== */
/*                            Macro definitions                               */
/* ========================================================================== */
#ifdef WITH_VMB_LOADER
   #define WITH_PROC_FS 1
#else
   #define WITH_OS_REGISTER 1
#endif /* WITH_VMB_LOADER */

/* ========================================================================== */
/*                              Local variables                               */
/* ========================================================================== */
typedef enum
{
   NOT_STARTED = 0,
   STARTED
} VMMC_LOADER_STATE_t;

typedef enum
{
   VMMC_LOADER_ERROR = (-1),
   VMMC_LOADER_SUCCESS = 0
} VMMC_LOADER_RETURN_t;

typedef struct
{
   /** state of vmmc loader image */
   VMMC_LOADER_STATE_t state;
   /** printing level filter */
   uint8_t trace_level;
#ifdef WITH_PROC_FS
   /** procfs directory handler */
   struct proc_dir_entry* proc_dir;
#endif /* WITH_PROC_FS */
#ifndef WITH_VMB_LOADER
   /** char device major number */
   uint16_t major_number;
#endif /* WITH_VMB_LOADER */
   /** cpu number for firmware from VMB */
   int8_t cpun;
   /** tc number for firmware from VMB */
   int8_t tc_num;
   /** pointer to memory block reserved for firmware */
   uint32_t* fw_memory_block;
   /** size of memory block reserved for firmware */
   uint32_t fw_memory_block_size;
   /** pointer to aligned firmware start address within the memory block */
   uint32_t* fw_adress;
   /** actual firmware size (bytes) */
   uint32_t fw_size;
} VMMC_LOADER_CONTEXT_t;

typedef struct
{
   /** CRC32 checksum */
   uint32_t crc32;
   /** start address for CRC32 checksum calculation */
   uint32_t st_addr_crc;
   /** end address for CRC32 checksum calculation */
   uint32_t en_addr_crc;
   /** firmware version */
   uint32_t fw_version;
   /** plain/encrypted: 0-plain, 1-encrypted */
   uint32_t encryption;
   /** magic word 0xCC123456 */
   uint32_t magic;
   /** firmware memory requirement */
   uint32_t fw_memory;
} VMMC_LOADER_FOOTER_t;

typedef struct
{
   /** langth of VOICE firmware image */
   uint32_t   length;
   /** pointer to VOICE firmware image */
   uint8_t*  data;
} VMMC_LOADER_DOWNLOAD_t;

/* ========================================================================== */
/*                           Function prototypes                              */
/* ========================================================================== */
#ifdef WITH_VMB_LOADER
extern int32_t drv_vmmc_loader_init(void);
extern void drv_vmmc_loader_cleanup(void);
#endif /* WITH_VMB_LOADER */

/**
   This function run voice firmware on CPU.

   \param fw_image pointer to VMMC_LOADER_DOWNLOAD_t structure

   \return int32_t IFX_SUCCESS or IFX_ERROR in case of error.
*/
extern int32_t drv_vmmc_loader_download_firmware(
   VMMC_LOADER_DOWNLOAD_t* fw_image);

/**
   This function stops voice firmware.
*/
extern void drv_vmmc_loader_stop_firmware(void);

#endif /* _DRV_VMMC_LOADER_HANDLER_H */
