#ifndef _DRV_VMMC_LOADER_LINUX_H
#define _DRV_VMMC_LOADER_LINUX_H
/******************************************************************************

                            Copyright (c) 2014-2015
                        Lantiq Beteiligungs-GmbH & Co.KG
                             http://www.lantiq.com

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

 ******************************************************************************/

/**
   \file drv_vmmc_loader_linux.h
   This file contains the implementation of linux specific parts
   of vmmc loader
*/

/* ========================================================================== */
/*                                 Includes                                   */
/* ========================================================================== */
#include <linux/fs.h>
#include <linux/kernel.h>

/* ========================================================================== */
/*                            Macro definitions                               */
/* ========================================================================== */
#define TRACE_ERROR        3
#define TRACE_INFO         2
#define TRACE_DEBUG        1

/** magic number */
#define VMMC_LOADER_MAGIC 'G'

/** download Firmware */
#define  FIO_LOADER_DOWNLOAD _IO(VMMC_LOADER_MAGIC, 7)

/* ========================================================================== */
/*                           Function prototypes                              */
/* ========================================================================== */
/**
   This function is called on vmmc loader open deivce.

   \param inode pointer to inode struct
   \param file pointer to file struct

   \return int32_t 0 in all cases
*/
extern int32_t drv_vmmc_loader_open (struct inode* inode, struct file* file_p);

/**
   This function is called on vmmc loader close deivce.

   \param inode pointer to inode struct
   \param file_p pointer to file struct

   \return int32_t 0 in all cases
*/
extern int32_t drv_vmmc_loader_close (struct inode* inode, struct file* filp);

/**
   This function print a message to console.

   \param level
   \param message
*/
extern void TRACE(uint8_t level, const char* message, ...);

#ifdef WITH_VMB_LOADER
extern int32_t drv_vmmc_loader_init_module(void);
extern void drv_vmmc_loader_cleanup_module(void);
#else
void __exit drv_vmmc_loader_cleanup_module (void);
int __init drv_vmmc_loader_init_module (void);
#endif
#endif /* _DRV_VMMC_LOADER_LINUX_H */

