#ifndef _drv_sdd_alm_priv_H
#define _drv_sdd_alm_priv_H
/******************************************************************************

                            Copyright (c) 2013-2015
                        Lantiq Beteiligungs-GmbH & Co.KG
                             http://www.lantiq.com

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

/**
   \file drv_sdd_alm_priv.h
   This file contains the implementation of the interrupt handler.
*/

/* ========================================================================== */
/*                                 Includes                                   */
/* ========================================================================== */
#include "drv_sdd_global.h"
#include "drv_sdd_fw_cmd.h"

/* ========================================================================== */
/*                             Type definitions                               */
/* ========================================================================== */
#ifdef SDD_FEAT_CAP_MEASURE
/* Capacitance measurement state */
typedef enum
{
  /* C-measurement is not active (initial) */
  SDD_CapMeasState_Inactive,
  /* C-measurement is started */
  SDD_CapMeasState_Started,
  /* C-measurement is in progress */
  SDD_CapMeasState_InProgress
} SDD_ALM_CAP_MEAS_STATE_t;
#endif /* SDD_FEAT_CAP_MEASURE */

/* Structure for the ALM channel including firmware message cache */
struct SDD_ALM
{
   /* Basic FW messages */
   SDD_TxRxGain_t          sdd_txrx_gain;

   /* Calibration FW message */
   SDD_Calibrate_t         fw_sdd_calibrate,
   /* cache for the last calibration results */
                           calibrationLastResults;

   /* SDD configuration FW messages */
   SDD_BasicConfig_t       fw_sdd_basic_config;
   /* temporary storage for basic configuration (required for MWL feature) */
   SDD_BasicConfig_t       sdd_basic_config_tmp_mwl;
   /* temporary storage for ring configuration (required for MWL feature) */
   SDD_RingConfig_t        sdd_ring_config_tmp_mwl;
#if defined(SDD_FEAT_GR909) || defined(SDD_FEAT_CAP_MEASURE)
   #ifndef S220
      SDD_GR909Config_t    sdd_gr909_config;
   #endif /* !S220 */
#endif

#ifdef S220
   /* Metering Pulse Control FW messages */
   struct TTX_GEN_CTRL     ttx_gen_ctrl;
#endif /* S220 */

#ifdef SDD_FEAT_AC_LEVEL_METERING
   /* AC level meter FW messages*/
   SDD_AcLevelMeterControl_t fw_sdd_acLevelMeterControl;
   SDD_AcLevelMeterConfig_t  fw_sdd_acLevelMeterConfig;
   SDD_AcLevelMeterResult_t  fw_sdd_acLevelMeterResult;
#endif

#ifdef SDD_FEAT_FXO
   /* FXO FW messages */
   SDD_FxoConfig_t         sdd_fxoConfig;
   SDD_FxoHookSwitch_t     sdd_fxoHookSwitch;

   /* the storage of battery, apoh, polarity and ring status for FXO line */
   volatile IFX_uint32_t   fxo_flags;
#endif /* SDD_FEAT_FXO */

#ifdef SDD_FEAT_CONT_MEASUREMENT
   /* Continuous measurement FW messages */
   SDD_ContMeasRead_t      fw_sdd_contMeasRead;
   #ifdef SDD_FEAT_CONT_MEASUREMENT_RESET
      SDD_ContMeasClear_t  fw_sdd_contMeasClear;
   #endif /* SDD_FEAT_CONT_MEASUREMENT_RESET */
#endif /* SDD_FEAT_CONT_MEASUREMENT */

   /* Cached FW message for SDD_Opmode */
   SDD_Opmode_t            sdd_opmode,
                           fw_last_sdd_opmode;

   /* Keep all FW messages above this line to make sure they are aligned to
      an even 32-bit address. The data below may have a different alignment. */

   /* Current line operating mode - reflects the actual opmode.
      Written in interrupt context by the firmware event,
      read by opmode change requesting thread to ensure that the
      opmode transition is complete */
   volatile IFX_uint8_t    curr_opmode;
#ifndef S220
   IFX_uint8_t             curr_polarity;
   IFX_uint8_t             curr_howler;
   IFX_uint8_t             curr_mwi_lamp;
#endif /* S220 */
   volatile IFX_boolean_t  bOpmodeChangePending;
   /* SDD event */
   IFXOS_event_t           sdd_event;

   IFX_TAPI_CALIBRATION_STATE_t
                           nCalibrationState;
   /* Indicates the current state of the FW */
   IFX_boolean_t           bCalibrationRunning;
   /* Indicates that calibration was called driver internal. */
   IFX_boolean_t           bCalibrationInternal;
   /* Indicates that calibration is needed after BBD download. */
   IFX_boolean_t           bCalibrationNeeded;
   /* Semaphore to defer until calibration has completed. */
   SDD_OS_mutex_t          mtxCalibrationWait;

   /* line type configured to FXS == IFX_TRUE or FXO == IFX_FALSE */
   IFX_boolean_t           line_type_fxs;
   /* Type of DC/DC converter */
   enum SDD_DCDC_TYPE      nDcDcType;

   /* Capacitance measurement feature */
#ifdef SDD_FEAT_CAP_MEASURE
   #ifdef S220
      SDD_CapacitanceMeasurementRead_t
                           fw_sdd_capacitance_meas;
   #else /* S220 */
      SDD_GR909PhoneDetection_t
                           fw_sdd_capacitance_meas;
   #endif /* S220 */

   /* State of capacitance measurement */
   volatile SDD_ALM_CAP_MEAS_STATE_t
                           eCapMeasState;
   volatile IFX_boolean_t  bCapMeasInProgress;
   /* IFX_TRUE - only Tip to Ring capacitance is measured. */
   IFX_boolean_t           bCapMeasTip2RingOnly;
#endif /* SDD_FEAT_CAP_MEASURE */

#ifdef SDD_FEAT_NLT
   /* Tip to Ring Capacitance measurement result. */
   IFX_TAPI_NLT_T2R_CAPACITANCE_RESULT_t
                           t2r_cap_result;
   /* Line to Ground Capacitance measurement result. */
   IFX_TAPI_NLT_L2GND_CAPACITANCE_RESULT_t
                           l2g_cap_result;

   /* open loop resistance configuration. */
   IFX_TAPI_NLT_OL_RESISTANCE_CONF_t
                           nlt_ResistanceConfig;
   /* open loop capacitance configuration. */
   IFX_TAPI_NLT_OL_CAPACITANCE_CONF_t
                           nlt_CapacitanceConfig;
   /** Configuration which Rmeas variant is connected to the chip. */
   IFX_TAPI_NLT_RMEAS_CFG_t
                           nRmeas;
#endif /* SDD_FEAT_NLT */

   /* elapsed time since last hook event */
   IFX_uint16_t            nLastHookEvtTimestamp;
   Timer_ID                nHookWindowTimerId;
   IFX_boolean_t           bHookWindow;
   /* Indicates override of the Standby DUP time configuration. */
   IFX_uint8_t             nStdbyDupTimeOverride;
};

#endif /* _drv_sdd_alm_priv_H */
