#ifndef _drv_sdd_api_H
#define _drv_sdd_api_H
/*******************************************************************************

                            Copyright (c) 2013-2016
                        Lantiq Beteiligungs-GmbH & Co.KG
                             http://www.lantiq.com

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

/**
   \file drv_sdd_api.h
   This file defines the SDD driver interface to be used by other drivers.
*/

/* ========================================================================== */
/*                                 Includes                                   */
/* ========================================================================== */
#include "ifx_types.h"
#include "drv_tapi_ll_interface.h"
#include "drv_sdd_io.h"

/* ========================================================================== */
/*                             Macro definitions                              */
/* ========================================================================== */
#define SDD_SUCCESS TAPI_SUCCESS

/* ========================================================================== */
/*                             Type definitions                               */
/* ========================================================================== */
struct SDD_VERSION_INFO {
   /* VCODEC FW version */
   IFX_uint8_t nVcodecFw[4];
   /* SDD (DCCtrl) version */
   IFX_uint8_t nSdd[4];
   /* ASDSP version */
   IFX_uint8_t nASDSP[4];
   /* SmartSLIC HW revision */
   IFX_uint8_t nSmartSLIC_HW_rev;
   /* SmartSLIC device ID */
   IFX_uint8_t nSmartSLIC_DevId;
#ifdef S220
   /* Device Number */
   IFX_uint32_t nSmartSLIC_DeviceNr;
   /* Calendar Week */
   IFX_uint8_t nSmartSLIC_CalendarWeek;
   /* Production Year */
   IFX_uint8_t nSmartSLIC_Year;
#endif /* S220 */
};

/* ========================================================================== */
/*                           Function prototypes                              */
/* ========================================================================== */

extern IFX_int32_t SDD_module_init(
                        void);

extern IFX_void_t  SDD_module_exit(
                        void);

extern IFX_TAPI_LL_DEV_t* SDD_TAPI_LL_DevicePrepare(
                        TAPI_DEV *pTapiDev,
                        IFX_uint32_t devNum);

extern IFX_TAPI_LL_CH_t* SDD_TAPI_LL_ChannelPrepare(
                        TAPI_CHANNEL *pTapiCh,
                        IFX_TAPI_LL_DEV_t *pLLDev,
                        IFX_uint32_t chNum);

extern IFX_int32_t SDD_TAPI_LL_DeviceInit(
                        IFX_TAPI_LL_DEV_t* pLLDev);

extern IFX_int32_t SDD_TAPI_LL_ChannelInit(
                        IFX_TAPI_LL_CH_t *pLLCh);

extern IFX_int32_t SDD_TAPI_LL_FW_Init(
                        IFX_TAPI_LL_DEV_t *pLLDev,
                        IFX_uint8_t nMode);

extern IFX_void_t SDD_TAPI_LL_DeviceExit(
                        IFX_TAPI_LL_DEV_t *pLLDev,
                        IFX_boolean_t bChipAccess);

extern IFX_void_t SDD_ALM_Opmode_Func_Register(
                        IFX_TAPI_DRV_CTX_ALM_t *pAlm);

extern IFX_int32_t SDD_ALM_BaseConf(
                        IFX_TAPI_LL_CH_t *pLLChannel);

extern IFX_void_t SDD_AddCaps(
                        IFX_TAPI_CAP_t *CapList,
                        IFX_uint8_t *pnCap,
                        IFX_TAPI_LL_DEV_t* pLLDev);

extern IFX_uint8_t SDD_ChannelCountGet(
                        IFX_TAPI_LL_DEV_t* pLLDev);

extern IFX_int32_t SDD_TAPI_LL_PCM_IF_Cfg (
                        IFX_TAPI_LL_DEV_t* pLLDev,
                        const IFX_TAPI_PCM_IF_CFG_t *pCfg);

extern IFX_int32_t SDD_TAPI_LL_PCM_ChCfg(
                        IFX_TAPI_LL_CH_t *pLLChannel,
                        IFX_TAPI_PCM_CFG_t const *pPCMConfig);

extern IFX_int32_t SDD_TAPI_LL_PCM_ChEnable(
                        IFX_TAPI_LL_CH_t *pLLCh,
                        IFX_uint32_t nMode,
                        IFX_TAPI_PCM_CFG_t *pPcmCfg);

extern IFX_int32_t SDD_TAPI_LL_ALM_Line_Type_Set(
                        IFX_TAPI_LL_CH_t *pLLChannel,
                        IFX_TAPI_LINE_TYPE_t nType);

extern IFX_int32_t SDD_Dev_Spec_Ioctl(
                        IFX_TAPI_LL_CH_t *pLLDummyCh,
                        IFX_uint32_t iocmd,
                        IFX_ulong_t  ioarg);

extern IFX_int32_t SDD_EXT_FwDownload(
                        IFX_TAPI_LL_DEV_t* pLLDev,
                        SDD_FW_Download_t *pSddFwDwld);

extern IFX_int32_t SDD_BBD_EXT_Download(
                        IFX_TAPI_LL_CH_t *pLLCh,
                        IFX_void_t *pBBD);

extern IFX_int32_t SDD_VersionGet(
                        IFX_TAPI_LL_DEV_t *pLLDev,
                        struct SDD_VERSION_INFO *pVersion);

#endif /* _drv_sdd_api_H */
