#ifndef _drv_sdd_fw_cmd_H
#define _drv_sdd_fw_cmd_H
/******************************************************************************

                           Copyright (c) 2014-2015
                        Lantiq Beteiligungs-GmbH & Co.KG
                             http://www.lantiq.com

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

/**
   \file drv_sdd_fw_cmd.h
   This file contains the SDD command messages.
*/

/* ========================================================================== */
/*                                 Includes                                   */
/* ========================================================================== */

/* ========================================================================== */
/*                             Macro definitions                              */
/* ========================================================================== */
#ifndef __PACKED__
   #if defined (__GNUC__) || defined (__GNUG__)
      /* GNU C or C++ compiler */
      #define __PACKED__ __attribute__ ((packed))
   #elif !defined (__PACKED__)
      #define __PACKED__      /* nothing */
   #endif
#endif

/* number of SDD command header bytes */
#define SDD_CMD_HEADER_CNT    4

#define CMD_HEAD_BE           \
   /** Read/Write */          \
   IFX_uint32_t RW : 1;       \
   /** Short Command */       \
   IFX_uint32_t SC : 1;       \
   /** Don't care */          \
   IFX_uint32_t BC : 1;       \
   /** Command */             \
   IFX_uint32_t CMD : 5;      \
   /** Reserved */            \
   IFX_uint32_t Res00 : 4;    \
   /** Channel */             \
   IFX_uint32_t CHAN : 4;     \
   /** Mode of Subcommand */  \
   IFX_uint32_t MOD : 3;      \
   /** EDSP Command */        \
   IFX_uint32_t ECMD : 5;     \
   /** Length */              \
   IFX_uint32_t LENGTH : 8

#define CMD_HEAD_LE           \
   /** Length */              \
   IFX_uint32_t LENGTH : 8;   \
   /** EDSP Command */        \
   IFX_uint32_t ECMD : 5;     \
   /** Mode of Subcommand */  \
   IFX_uint32_t MOD : 3;      \
   /** Channel */             \
   IFX_uint32_t CHAN : 4;     \
   /** Reserved */            \
   IFX_uint32_t Res00 : 4;    \
   /** Command */             \
   IFX_uint32_t CMD : 5;      \
   /** Don't care */          \
   IFX_uint32_t BC : 1;       \
   /** Short Command */       \
   IFX_uint32_t SC : 1;       \
   /** Read/Write */          \
   IFX_uint32_t RW : 1

#define CMD_SDD      4
#define CMD_EOP      6
#define MOD_PCM      0
#define MOD_SDD      0
#define MOD_SDD1     1
#define MOD_SIG_GEN  3
#define MOD_SYS      7

#define CMDREAD      1
#define CMDWRITE     0
#define CMD_HDR_CNT  4

#define CMD_CHAN_A   0
#define CMD_CHAN_B   1


/* ========================================================================== */
/*                             Type definitions                               */
/* ========================================================================== */

/** The VCODEC Firmware Version command allows the control CPU to read the
    version of the VCODEC firmware. */
struct SYS_VCODEC_FW_Version
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Major */
   IFX_uint32_t MAJ : 8;
   /** Minor */
   IFX_uint32_t MIN : 8;
   /** Hotfix */
   IFX_uint32_t HF : 8;
   /** Platform */
   IFX_uint32_t PLA : 8;
   /** Time Stamp */
   IFX_uint32_t TIME : 32;
#else
   CMD_HEAD_LE;
   /** Platform */
   IFX_uint32_t PLA : 8;
   /** Hotfix */
   IFX_uint32_t HF : 8;
   /** Minor */
   IFX_uint32_t MIN : 8;
   /** Major */
   IFX_uint32_t MAJ : 8;
   /** Time Stamp */
   IFX_uint32_t TIME : 32;
#endif
} __PACKED__;
#define SYS_VCODEC_FW_Version_ECMD    6
#define SYS_VCODEC_FW_Version_LENGTH  8
/** Mask for Test Firmware flag in Major number field */
#define SYS_VCODEC_FW_Version_MAJ_MASK_TFW 0x80

/** The VCODEC Read Capabilities command reads the capabilities of the
    firmware variant. */
struct SYS_ReadCapabilities
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   #ifdef S220
   /** Reserved */
   IFX_uint32_t Res01 : 31;
   /** GR909 available */
   IFX_uint32_t GR909 : 1;
   #else
   /** Reserved */
   IFX_uint32_t Res01 : 22;
   /** PCM Capability */
   IFX_uint32_t CapPcm : 1;
   /** Buck bust DCDC converter is not supported */
   IFX_uint32_t CapBB : 1;
   /** Wide Band PCM Time Slot Variant Capability */
   IFX_uint32_t CapWbTsC : 1;
   /** Combined DCDC Converter Capability */
   IFX_uint32_t CapCombDcDc : 1;
   /** Extended Line Testing Capability is not supported */
   IFX_uint32_t CapExtLT : 1;
   /** Capacitance Measurement Capability is not supported */
   IFX_uint32_t CapCMeas : 1;
   /** Download of AsDsp Code Capability is not supported */
   IFX_uint32_t CapAsDspDL : 1;
   /** DTMF Receiver Capability is not supported */
   IFX_uint32_t CapDTMF : 1;
   /** Message Waiting Indication Capability */
   IFX_uint32_t CapMWI : 1;
   /** Auto Sleep Capability */
   IFX_uint32_t CapAS : 1;
   #endif /* S220 */
#else
   CMD_HEAD_LE;
   #ifdef S220
   /** GR909 available */
   IFX_uint32_t GR909 : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 31;
   #else
   /** Auto Sleep Capability */
   IFX_uint32_t CapAS : 1;
   /** Message Waiting Indication Capability */
   IFX_uint32_t CapMWI : 1;
   /** DTMF Receiver Capability is not supported */
   IFX_uint32_t CapDTMF : 1;
   /** Download of AsDsp Code Capability is not supported */
   IFX_uint32_t CapAsDspDL : 1;
   /** Capacitance Measurement Capability is not supported */
   IFX_uint32_t CapCMeas : 1;
   /** Extended Line Testing Capability is not supported */
   IFX_uint32_t CapExtLT : 1;
   /** Combined DCDC Converter Capability */
   IFX_uint32_t CapCombDcDc : 1;
   /** Wide Band PCM Time Slot Variant Capability */
   IFX_uint32_t CapWbTsC : 1;
   /** Buck bust DCDC converter is not supported */
   IFX_uint32_t CapBB : 1;
   /** PCM Capability */
   IFX_uint32_t CapPcm : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 22;
   # endif /* S220 */
#endif
} __PACKED__;
#define SYS_ReadCapabilities_ECMD    7
#define SYS_ReadCapabilities_LENGTH  4
#ifdef S220
#define SYS_ReadCapabilities_GR909_NotAvail    0
#define SYS_ReadCapabilities_GR909_Avail       1

/** The Control message enables system specific configurations */
struct SYS_CTRL
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Reserved */
   IFX_uint32_t Res01 : 27;
   /** Charge Pump Voltage Level */
   IFX_uint32_t CP_VOLT : 3;
   /** Charge Pump Enable */
   IFX_uint32_t CP_EN : 1;
   /** Sleep Enable */
   IFX_uint32_t SE : 1;
#else
   CMD_HEAD_LE;
   /** Sleep Enable */
   IFX_uint32_t SE : 1;
   /** Charge Pump Enable */
   IFX_uint32_t CP_EN : 1;
   /** Charge Pump Voltage Level */
   IFX_uint32_t CP_VOLT : 3;
   /** Reserved */
   IFX_uint32_t Res01 : 27;
#endif
} __PACKED__;
#define SYS_CTRL_ECMD            8
#define SYS_CTRL_LENGTH          4
#define SYS_CTRL_CP_VOLT_3_95V   0
#define SYS_CTRL_CP_VOLT_4_3V    1
#define SYS_CTRL_CP_VOLT_4_65V   2
#define SYS_CTRL_CP_VOLT_5_0V    3
#define SYS_CTRL_CP_VOLT_5_35V   4
#define SYS_CTRL_CP_VOLT_5_7V    5
#define SYS_CTRL_CP_VOLT_6_05V   6
#define SYS_CTRL_CP_VOLT_6_4V    7
#endif /* S220 */

/** The Smart SLIC Status message allows the status of the attached
    Smart SLIC devices to be read. */
struct SYS_SmartSlicStatus
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Reserved */
   IFX_uint32_t Res01 : 14;
   /** Number of available analog line channels */
   IFX_uint32_t NALI : 2;
   /** Reserved */
   IFX_uint32_t Res02 : 8;
   /** Smart SLIC Status */
   IFX_uint32_t SLST : 8;
#else
   CMD_HEAD_LE;
   /** Smart SLIC Status */
   IFX_uint32_t SLST : 8;
   /** Reserved */
   IFX_uint32_t Res02 : 8;
   /** Number of available analog line channels */
   IFX_uint32_t NALI : 2;
   /** Reserved */
   IFX_uint32_t Res01 : 14;
#endif
} __PACKED__;
#define SYS_SmartSlicStatus_ECMD   14
#define SYS_SmartSlicStatus_LENGTH 4
#define SYS_SmartSlicStatus_SLST_NoSLIC 0

#ifdef S220
/** The Command Error Acknowledge has to be sent after the DART XS
    has reported a Command Error Event */
struct SYS_CMD_ERR_ACK
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
#else
   CMD_HEAD_LE;
#endif
} __PACKED__;
#define SYS_CMD_ERR_ACK_ECMD    0
#define SYS_CMD_ERR_ACK_LENGTH  0

/** The Metering Pulse Control starts (and stops) the metering pulse
    generator for one single metering pulse. The pulse length can be
    programmed via the message SDD_BasicConfig */
struct TTX_GEN_CTRL
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Enable */
   IFX_uint32_t EN : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 31;
#else
   CMD_HEAD_LE;
   /** Reserved */
   IFX_uint32_t Res01 : 31;
   /** Enable */
   IFX_uint32_t EN : 1;
#endif
} __PACKED__;
#define TTX_GEN_CTRL_ECMD    4
#define TTX_GEN_CTRL_LENGTH  4
#define TTX_GEN_CTRL_DIS     0
#define TTX_GEN_CTRL_EN      1

#endif /* S220 */

/** The SDD_RevisionRead message is used to read revision information
    from the Smart Device Driver. */
typedef struct SDD_RevisionRead
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   #ifdef S220
   /** Device Number */
   IFX_uint32_t DEVNR : 11;
   /** Calendar Week */
   IFX_uint32_t CW : 6;
   /** Production Year */
   IFX_uint32_t YEAR : 6;
   /** Channel */
   IFX_uint32_t CH : 1;
   /** Device ID Representing PEB Number */
   IFX_uint32_t DEV : 8;
   /** ASDSP FW Version Number */
   IFX_uint32_t ASDSPVers : 8;
   /** ASDSP FW Version Number */
   IFX_uint32_t ASDSPStep : 8;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   #else
   /** DC/DC Hardware */
   IFX_uint32_t DcDcHardware : 4;
   /** Hardware Revision */
   IFX_uint32_t HardwareRevision : 4;
   /** Used Firmware Revision */
   IFX_uint32_t DCCtrlVers : 8;
   /** Reserved */
   IFX_uint32_t Res01 : 1;
   /** Device ID Representing PEB Number */
   IFX_uint32_t DEV : 7;
   /** Number of channels (FXO and FXS) */
   IFX_uint32_t CH : 3;
   /** FXO functionality in channel 1 */
   IFX_uint32_t FXO1 : 1;
   /** FXO functionality in channel 0 */
   IFX_uint32_t FXO0 : 1;
   /** SLIC type */
   IFX_uint32_t SLIC : 2;
   /** Reserved */
   IFX_uint32_t PROT : 1;
   /** ASDSP FW Version Number */
   IFX_uint32_t ASDSPVers : 8;
   /** ASDSP FW Version Number */
   IFX_uint32_t ASDSPStep : 8;
   /** HotFix Revision Number */
   IFX_uint32_t DCCtrlHotFixRevision : 8;
   /** Engineering Revision Number */
   IFX_uint32_t DCCtrlVersStep : 8;
   #endif /* S220 */
#else
   CMD_HEAD_LE;
   #ifdef S220
   /** Device ID Representing PEB Number */
   IFX_uint32_t DEV : 8;
   /** Channel */
   IFX_uint32_t CH : 1;
   /** Production Year */
   IFX_uint32_t YEAR : 6;
   /** Calendar Week */
   IFX_uint32_t CW : 6;
   /** Device Number */
   IFX_uint32_t DEVNR : 11;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   /** ASDSP FW Version Number */
   IFX_uint32_t ASDSPStep : 8;
   /** ASDSP FW Version Number */
   IFX_uint32_t ASDSPVers : 8;
   #else
   /** Reserved */
   IFX_uint32_t PROT : 1;
   /** SLIC type */
   IFX_uint32_t SLIC : 2;
   /** FXO functionality in channel 0 */
   IFX_uint32_t FXO0 : 1;
   /** FXO functionality in channel 1 */
   IFX_uint32_t FXO1 : 1;
   /** Number of channels (FXO and FXS) */
   IFX_uint32_t CH : 3;
   /** Device ID Representing PEB Number */
   IFX_uint32_t DEV : 7;
   /** Reserved */
   IFX_uint32_t Res01 : 1;
   /** Used Firmware Revision */
   IFX_uint32_t DCCtrlVers : 8;
   /** Hardware Revision */
   IFX_uint32_t HardwareRevision : 4;
   /** DC/DC Hardware */
   IFX_uint32_t DcDcHardware : 4;
   /** Engineering Revision Number */
   IFX_uint32_t DCCtrlVersStep : 8;
   /** HotFix Revision Number */
   IFX_uint32_t DCCtrlHotFixRevision : 8;
   /** ASDSP FW Version Number */
   IFX_uint32_t ASDSPStep : 8;
   /** ASDSP FW Version Number */
   IFX_uint32_t ASDSPVers : 8;
   #endif /* S220 */
#endif
} __PACKED__ SDD_RevisionRead_t;
#define SDD_RevisionRead_ECMD    0
#define SDD_RevisionRead_LENGTH  8
#define SDD_RevisionRead_DcDcHardware_Mask_Combined 1

/** The SDD_ContMeasClear message is used to clear all the continuous
    measurement results. */
typedef struct SDD_ContMeasClear
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Reserved */
   IFX_uint32_t Res01 : 15;
   /** Clear Continuous Measurement Data */
   IFX_uint32_t ClearCMD : 1;
   /** Reserved */
   IFX_uint32_t Res02 : 16;
#else
   CMD_HEAD_LE;
   /** Reserved */
   IFX_uint32_t Res02 : 16;
   /** Clear Continuous Measurement Data */
   IFX_uint32_t ClearCMD : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 15;
#endif
} __PACKED__ SDD_ContMeasClear_t;
#define SDD_ContMeasClear_ECMD    1
#define SDD_ContMeasClear_LENGTH  4
#define SDD_ContMeasClear_NO_CLEAR 0
#define SDD_ContMeasClear_CLEAR    1

/** The SDD_ContMeasRead message is used to read all the results from
    the continuous measurement block. */
typedef struct SDD_ContMeasRead
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   #ifdef S220
   /** Line Voltage */
   IFX_uint32_t Vlim : 16;
   /** Line Current */
   IFX_uint32_t Itrans : 16;
   /** RING Current */
   IFX_uint32_t Iring : 16;
   /** RING Voltage */
   IFX_uint32_t Vring : 16;
   /** DCDC Lock information */
   IFX_uint32_t DcDcLock : 1;
   /** Charge Pump Status Bit */
   IFX_uint32_t CpStat : 1;
   /** Reserved */
   IFX_uint32_t Res02 : 4;
   /** DCDC Frequency */
   IFX_uint32_t DcDcFreq : 10;
   /** Reserved */
   IFX_uint32_t Res03 : 16;
   #else
   /** Line Voltage on RING Wire */
   IFX_uint32_t VlineRing : 16;
   /** Line Voltage on TIP Wire */
   IFX_uint32_t VlineTip : 16;
   /** Desired Line Voltage */
   IFX_uint32_t Vlim : 16;
   /** Line Current */
   IFX_uint32_t Itrans : 16;
   /** Longitudinal Current */
   IFX_uint32_t Ilong : 16;
   /** SLIC Temperature */
   IFX_uint32_t SlicTemp : 16;
   /** RING Current */
   IFX_uint32_t Iring : 16;
   /** RING Voltage */
   IFX_uint32_t Vring : 16;
   /** TTX Adaptation Coefficient Real Part */
   IFX_uint32_t TtxReal : 16;
   /** TTX Adaptation Coefficient Imaginary Part */
   IFX_uint32_t TtxImag : 16;
   /** TTX Burst Length */
   IFX_uint32_t TtxLen : 16;
   /** TTX Current */
   IFX_uint32_t Ittx : 16;
   /** TTX Voltage */
   IFX_uint32_t Vttx : 16;
   /** Battery Voltage */
   IFX_uint32_t Vbat : 16;
   #endif /* S220 */
#else
   CMD_HEAD_LE;
   #ifdef S220
   /** Line Current */
   IFX_uint32_t Itrans : 16;
   /** Line Voltage */
   IFX_uint32_t Vlim : 16;
   /** RING Voltage */
   IFX_uint32_t Vring : 16;
   /** RING Current */
   IFX_uint32_t Iring : 16;
   /** Reserved */
   IFX_uint32_t Res03 : 16;
   /** DCDC Frequency */
   IFX_uint32_t DcDcFreq : 10;
   /** Reserved */
   IFX_uint32_t Res02 : 4;
   /** Charge Pump Status Bit */
   IFX_uint32_t CpStat : 1;
   /** DCDC Lock information */
   IFX_uint32_t DcDcLock : 1;
   #else
   /** Line Voltage on TIP Wire */
   IFX_uint32_t VlineTip : 16;
   /** Line Voltage on RING Wire */
   IFX_uint32_t VlineRing : 16;
   /** Line Current */
   IFX_uint32_t Itrans : 16;
   /** Desired Line Voltage */
   IFX_uint32_t Vlim : 16;
   /** SLIC Temperature */
   IFX_uint32_t SlicTemp : 16;
   /** Longitudinal Current */
   IFX_uint32_t Ilong : 16;
   /** RING Voltage */
   IFX_uint32_t Vring : 16;
   /** RING Current */
   IFX_uint32_t Iring : 16;
   /** TTX Adaptation Coefficient Imaginary Part */
   IFX_uint32_t TtxImag : 16;
   /** TTX Adaptation Coefficient Real Part */
   IFX_uint32_t TtxReal : 16;
   /** TTX Current */
   IFX_uint32_t Ittx : 16;
   /** TTX Burst Length */
   IFX_uint32_t TtxLen : 16;
   /** Battery Voltage */
   IFX_uint32_t Vbat : 16;
   /** TTX Voltage */
   IFX_uint32_t Vttx : 16;
   #endif /* S220 */
#endif
} __PACKED__ SDD_ContMeasRead_t;
#define SDD_ContMeasRead_ECMD   2
#ifdef S220
#define SDD_ContMeasRead_LENGTH 12
#else /* S220 */
#define SDD_ContMeasRead_LENGTH 28
#endif /* S220 */

/** The SDD_Calibrate message is used to read calibration data. */
typedef struct SDD_Calibrate
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   #ifdef S220
   /** TX Path Offset */
   IFX_uint32_t TxOffset : 16;
   /** IDAC Gain Correction */
   IFX_uint32_t IdacGain : 16;
   /** Ring Current Offset Compensation */
   IFX_uint32_t RingOffset : 16;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   #else /* S220 */
   /** TX Path Offset */
   IFX_uint32_t TxOffset : 16;
   /** Measurement Equipment Offset */
   IFX_uint32_t MeOffset : 16;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   /** Reserved */
   IFX_uint32_t Res02 : 16;
   /** IDAC Gain Correction */
   IFX_uint32_t IdacGain : 16;
   /** Reserved */
   IFX_uint32_t Res03 : 16;
   /** External Measurment Resitor */
   IFX_uint32_t ExtMeasRes : 16;
   /** Combined DC/DC Voltage Correction */
   IFX_uint32_t Vdcdc : 16;
   #endif /* S220 */
#else
   CMD_HEAD_LE;
   #ifdef S220
   /** IDAC Gain Correction */
   IFX_uint32_t IdacGain : 16;
   /** TX Path Offset */
   IFX_uint32_t TxOffset : 16;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   /** Ring Current Offset Compensation */
   IFX_uint32_t RingOffset : 16;
   #else /* S220 */
   /** Measurement Equipment Offset */
   IFX_uint32_t MeOffset : 16;
   /** TX Path Offset */
   IFX_uint32_t TxOffset : 16;
   /** Reserved */
   IFX_uint32_t Res02 : 16;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   /** Reserved */
   IFX_uint32_t Res03 : 16;
   /** IDAC Gain Correction */
   IFX_uint32_t IdacGain : 16;
   /** Combined DC/DC Voltage Correction */
   IFX_uint32_t Vdcdc : 16;
   /** External Measurment Resitor */
   IFX_uint32_t ExtMeasRes : 16;
   #endif /* S220 */
#endif
} __PACKED__ SDD_Calibrate_t;
#define SDD_Calibrate_ECMD    3
#ifdef S220
#define SDD_Calibrate_LENGTH  8
#else /* S220 */
#define SDD_Calibrate_LENGTH  16
#endif /* S220 */

/** The SDD_BasicConfig messages are used to write and read all the
    basic parameters for the SMART Device Driver */
typedef struct SDD_BasicConfig
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   #ifdef S220
   /** Reserved */
   IFX_uint32_t Res01 : 4;
   /** DUP Time for Hook Debouncing in ACT Mode */
   IFX_uint32_t ActiveDup : 4;
   /** DUP Time for Hook Debouncing in GS Mode */
   IFX_uint32_t GsDup : 4;
   /** Reserved */
   IFX_uint32_t Res02 : 4;
   /** DUP Time for Ground Key Debouncing */
   IFX_uint32_t GndkDup : 4;
   /** Emergency Shut Down Debounce Time */
   IFX_uint32_t EsdDup : 4;
   /** Reserved */
   IFX_uint32_t Res03 : 2;
   /** Automatic Sense Bias Enable */
   IFX_uint32_t AutoBiasEn : 1;
   /** Reserved */
   IFX_uint32_t Res04 : 5;
   /** Combined Low Power Offhook Voltage */
   IFX_uint32_t CLPOffhook : 4;
   /** DC/DC Overhead Voltage */
   IFX_uint32_t DcDcOvh : 4;
   /** Combined Low Power Overhead Voltage */
   IFX_uint32_t CLPOvh : 5;
   /** Standby Voltage */
   IFX_uint32_t StbyVolt : 2;
   /** Reserved */
   IFX_uint32_t Res05 : 1;
   /** TTX Burst Length */
   IFX_uint32_t TtxBurstLength : 16;
   /** Current gain for PID regulator */
   IFX_uint32_t DcPidGain : 16;
   /** On-Hook Threshold in ACT Mode */
   IFX_uint32_t ActOnhookThresh : 16;
   /** Off-Hook Threshold in ACT Mode */
   IFX_uint32_t ActOffhookThresh : 16;
   /** Reserved */
   IFX_uint32_t Res06 : 16;
   /** Open Loop Voltage Limit */
   IFX_uint32_t VoltageLimit : 16;
   /** Closed Loop Current Limit */
   IFX_uint32_t CurrentLimit : 16;
   #else /* S220 */
   /** DUP Time for Hook Debouncing in ONHOOK Mode */
   IFX_uint32_t OnhookDup : 4;
   /** DUP Time for Hook Debouncing in ACT Mode */
   IFX_uint32_t ActiveDup : 4;
   /** DUP Time for Hook Debouncing in GS Mode */
   IFX_uint32_t GsDup : 4;
   /** n.a. */
   IFX_uint32_t SoftDup : 4;
   /** DUP Time for Ground Key Debouncing  */
   IFX_uint32_t GndkDup : 4;
   /** DUP Time for Excessive Temperature Detection Debouncing */
   IFX_uint32_t OvtDup : 4;
   /** Ground Start Enable Bit */
   IFX_uint32_t GsEn : 1;
   /** n.a. */
   IFX_uint32_t FastTran : 1;
   /** Automatic Sense Bias Enable */
   IFX_uint32_t AutoBiasEn : 1;
   /** n.a. */
   IFX_uint32_t RpSoft : 1;
   /** Idle Extension Bit */
   IFX_uint32_t IdleExt : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 1;
   /** Disable Buffer Current Limit for SmartSLIC P */
   IFX_uint32_t BufferLimitDis : 1;
   /** Disable automatic mute of transmit samples during ringing */
   IFX_uint32_t TxMuteDis : 1;
   /** Voltage Reduction for Standby Low Power */
   IFX_uint32_t LPVoltageOffset : 4;
   /** DC/DC Overhead Voltage */
   IFX_uint32_t DcDcOvh : 4;
   /** Reserved */
   IFX_uint32_t Res02 : 4;
   /** Automatic Transition from Ground Fault to Disconnect Enable */
   IFX_uint32_t GrFaultDisEn : 1;
   /** Reserved */
   IFX_uint32_t Res03 : 2;
   /** Low Power Bias */
   IFX_uint32_t LpBias : 1;
   /** TTX Burst Length */
   IFX_uint32_t TtxBurstLength : 16;
   /** n.a. */
   IFX_uint32_t OnhookThresh : 16;
   /** On-Hook Threshold in ACT Mode */
   IFX_uint32_t ActOnhookThresh : 16;
   /** Off-Hook Threshold in ACT Mode */
   IFX_uint32_t ActOffhookThresh : 16;
   /** Hook Threshold in GS Mode */
   IFX_uint32_t GsThresh : 16;
   /** Open Loop Voltage Limit */
   IFX_uint32_t VoltageLimit : 16;
   /** Closed Loop Current Limit */
   IFX_uint32_t CurrentLimit : 16;
   /** Threshold for Ground Key (Low) */
   IFX_uint32_t GkLowThresh : 16;
   /** Threshold for Ground Key (High) */
   IFX_uint32_t GkHighThresh : 16;
   #endif /* S220 */
#else
   CMD_HEAD_LE;
   #ifdef S220
   /** Reserved */
   IFX_uint32_t Res04 : 5;
   /** Automatic Sense Bias Enable */
   IFX_uint32_t AutoBiasEn : 1;
   /** Reserved */
   IFX_uint32_t Res03 : 2;
   /** Emergency Shut Down Debounce Time */
   IFX_uint32_t EsdDup : 4;
   /** DUP Time for Ground Key Debouncing */
   IFX_uint32_t GndkDup : 4;
   /** Reserved */
   IFX_uint32_t Res02 : 4;
   /** DUP Time for Hook Debouncing in GS Mode */
   IFX_uint32_t GsDup : 4;
   /** DUP Time for Hook Debouncing in ACT Mode */
   IFX_uint32_t ActiveDup : 4;
   /** Reserved */
   IFX_uint32_t Res01 : 4;
   /** TTX Burst Length */
   IFX_uint32_t TtxBurstLength : 16;
   /** Reserved */
   IFX_uint32_t Res05 : 1;
   /** Standby Voltage */
   IFX_uint32_t StbyVolt : 2;
   /** Combined Low Power Overhead Voltage */
   IFX_uint32_t CLPOvh : 5;
   /** DC/DC Overhead Voltage */
   IFX_uint32_t DcDcOvh : 4;
   /** Combined Low Power Offhook Voltage */
   IFX_uint32_t CLPOffhook : 4;
   /** On-Hook Threshold in ACT Mode */
   IFX_uint32_t ActOnhookThresh : 16;
   /** Current gain for PID regulator */
   IFX_uint32_t DcPidGain : 16;
   /** Reserved */
   IFX_uint32_t Res06 : 16;
   /** Off-Hook Threshold in ACT Mode */
   IFX_uint32_t ActOffhookThresh : 16;
   /** Closed Loop Current Limit */
   IFX_uint32_t CurrentLimit : 16;
   /** Open Loop Voltage Limit */
   IFX_uint32_t VoltageLimit : 16;
   #else /* S220 */
   /** Disable automatic mute of transmit samples during ringing */
   IFX_uint32_t TxMuteDis : 1;
   /** Disable Buffer Current Limit for SmartSLIC P */
   IFX_uint32_t BufferLimitDis : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 1;
   /** Idle Extension Bit */
   IFX_uint32_t IdleExt : 1;
   /** n.a. */
   IFX_uint32_t RpSoft : 1;
   /** Automatic Sense Bias Enable */
   IFX_uint32_t AutoBiasEn : 1;
   /** n.a. */
   IFX_uint32_t FastTran : 1;
   /** Ground Start Enable Bit */
   IFX_uint32_t GsEn : 1;
   /** DUP Time for Excessive Temperature Detection Debouncing */
   IFX_uint32_t OvtDup : 4;
   /** DUP Time for Ground Key Debouncing  */
   IFX_uint32_t GndkDup : 4;
   /** n.a. */
   IFX_uint32_t SoftDup : 4;
   /** DUP Time for Hook Debouncing in GS Mode */
   IFX_uint32_t GsDup : 4;
   /** DUP Time for Hook Debouncing in ACT Mode */
   IFX_uint32_t ActiveDup : 4;
   /** DUP Time for Hook Debouncing in ONHOOK Mode */
   IFX_uint32_t OnhookDup : 4;
   /** TTX Burst Length */
   IFX_uint32_t TtxBurstLength : 16;
   /** Low Power Bias */
   IFX_uint32_t LpBias : 1;
   /** Reserved */
   IFX_uint32_t Res03 : 2;
   /** Automatic Transition from Ground Fault to Disconnect Enable */
   IFX_uint32_t GrFaultDisEn : 1;
   /** Reserved */
   IFX_uint32_t Res02 : 4;
   /** DC/DC Overhead Voltage */
   IFX_uint32_t DcDcOvh : 4;
   /** Voltage Reduction for Standby Low Power */
   IFX_uint32_t LPVoltageOffset : 4;
   /** On-Hook Threshold in ACT Mode */
   IFX_uint32_t ActOnhookThresh : 16;
   /** n.a. */
   IFX_uint32_t OnhookThresh : 16;
   /** Hoook Threshold in GS Mode */
   IFX_uint32_t GsThresh : 16;
   /** Off-Hook Threshold in ACT Mode */
   IFX_uint32_t ActOffhookThresh : 16;
   /** Closed Loop Current Limit */
   IFX_uint32_t CurrentLimit : 16;
   /** Open Loop Voltage Limit */
   IFX_uint32_t VoltageLimit : 16;
   /** Threshold for Ground Key (High) */
   IFX_uint32_t GkHighThresh : 16;
   /** Threshold for Ground Key (Low) */
   IFX_uint32_t GkLowThresh : 16;
   #endif /* S220 */
#endif
} __PACKED__ SDD_BasicConfig_t;
#define SDD_BasicConfig_ECMD    4
#ifdef S220
#define SDD_BasicConfig_LENGTH  20
#else /* S220 */
#define SDD_BasicConfig_LENGTH  24
#endif /* S220 */
#define SDD_BasicConfig_ENABLE  1
#define SDD_BasicConfig_DISABLE 0

/** The SDD_RingConfig messages are used to write and read the parameters
    used for the ringing mode of the SMART Device Driver. */
typedef struct SDD_RingConfig
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   #ifdef S220
   /** Reserved */
   IFX_uint32_t Res01 : 2;
   /**  Ring Trip Type */
   IFX_uint32_t RingTripType : 2;
   /** Wave Form of Ringing Signal */
   IFX_uint32_t RingSignal : 1;
   /** Ring Crest Factor */
   IFX_uint32_t RingCrestFact : 3;
   /** Reserved */
   IFX_uint32_t Res02 : 8;
   /** Ringing Frequency */
   IFX_uint32_t Frequency : 16;
   /** Ringing Amplitude */
   IFX_uint32_t Amplitude : 16;
   /** Ringing Hook Threshold */
   IFX_uint32_t Thresh : 16;
   /** Ringing DC Offset */
   IFX_uint32_t DcOffset : 16;
   /** Maximum Ring Current */
   IFX_uint32_t Imax : 16;
   /** Ringing Regulation Coefficient */
   IFX_uint32_t RegCoeff : 16;
   /** Minimum Ringing Voltage */
   IFX_uint32_t Vmin : 16;
   /** Rserved */
   IFX_uint32_t Res03 : 16;
   /** Fast Hook Threshold */
   IFX_uint32_t FastThresh : 16;
   #else /* S220 */
   /** Ringing MODE */
   IFX_uint32_t RingMode : 2;
   /**  Ring Trip Type */
   IFX_uint32_t RingTripType : 2;
   /** Ringing Signal Form */
   IFX_uint32_t RingSignal : 1;
   /** Ring Crest Factor */
   IFX_uint32_t RingCrestFact : 3;
   /** Ring Delay */
   IFX_uint32_t RingDelay : 8;
   /** Ringing Frequency */
   IFX_uint32_t RingFrequency : 16;
   /** Ringing Amplitude */
   IFX_uint32_t RingAmplitude : 16;
   /** Ringing Hook Threshold */
   IFX_uint32_t RingThresh : 16;
   /** Ringing DC Offset */
   IFX_uint32_t RingDcOffset : 16;
   /** Maximum Ring Current */
   IFX_uint32_t RingImax : 16;
   /** Ringing Regulation Coefficient */
   IFX_uint32_t RingCoeff : 16;
   /** Minimum Ringing Voltage (peak) */
   IFX_uint32_t RingVmin : 16;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   /** Fast Hook Threshold */
   IFX_uint32_t FastThresh : 16;
   #endif /* S220 */
#else
   CMD_HEAD_LE;
   #ifdef S220
   /** Ringing Frequency */
   IFX_uint32_t Frequency : 16;
   /** Reserved */
   IFX_uint32_t Res02 : 8;
   /** Ring Crest Factor */
   IFX_uint32_t RingCrestFact : 3;
   /** Wave Form of Ringing Signal */
   IFX_uint32_t RingSignal : 1;
   /**  Ring Trip Type */
   IFX_uint32_t RingTripType : 2;
   /** Reserved */
   IFX_uint32_t Res01 : 2;
   /** Ringing Hook Threshold */
   IFX_uint32_t Thresh : 16;
   /** Ringing Amplitude */
   IFX_uint32_t Amplitude : 16;
   /** Maximum Ring Current */
   IFX_uint32_t Imax : 16;
   /** Ringing DC Offset */
   IFX_uint32_t DcOffset : 16;
   /** Minimum Ringing Voltage */
   IFX_uint32_t Vmin : 16;
   /** Ringing Regulation Coefficient */
   IFX_uint32_t RegCoeff : 16;
   /** Fast Hook Threshold */
   IFX_uint32_t FastThresh : 16;
   /** Rserved */
   IFX_uint32_t Res03 : 16;
   #else /* S220 */
   /** Ringing Frequency */
   IFX_uint32_t RingFrequency : 16;
   /** Ring Delay */
   IFX_uint32_t RingDelay : 8;
   /** Ring Crest Factor */
   IFX_uint32_t RingCrestFact : 3;
   /** Ringing Signal Form */
   IFX_uint32_t RingSignal : 1;
   /**  Ring Trip Type */
   IFX_uint32_t RingTripType : 2;
   /** Ringing MODE */
   IFX_uint32_t RingMode : 2;
   /** Ringing Hook Threshold */
   IFX_uint32_t RingThresh : 16;
   /** Ringing Amplitude */
   IFX_uint32_t RingAmplitude : 16;
   /** Maximum Ring Current */
   IFX_uint32_t RingImax : 16;
   /** Ringing DC Offset */
   IFX_uint32_t RingDcOffset : 16;
   /** Minimum Ringing Voltage (peak) */
   IFX_uint32_t RingVmin : 16;
   /** Ringing Regulation Coefficient */
   IFX_uint32_t RingCoeff : 16;
   /** Fast Hook Threshold */
   IFX_uint32_t FastThresh : 16;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   #endif /* S220 */
#endif
} __PACKED__ SDD_RingConfig_t;
#define SDD_RingConfig_ECMD   5
#define SDD_RingConfig_LENGTH 0x14
#define SDD_RingConfig_DC_Ring_Trip   0
#define SDD_RingConfig_AC_Ring_Trip   1
#define SDD_RingConfig_Fast_Ring_Trip 2
#define SDD_RingConfig_Sinewave       0
#ifdef S220
#define SDD_RingConfig_Trapez 1
#define SDD_RingConfig_RingCrestFactor_1_0   0
#define SDD_RingConfig_RingCrestFactor_1_10  1
#define SDD_RingConfig_RingCrestFactor_1_21  2
#define SDD_RingConfig_RingCrestFactor_1_31  3
#define SDD_RingConfig_RingCrestFactor_1_42  4
#define SDD_RingConfig_RingCrestFactor_1_52  5
#define SDD_RingConfig_RingCrestFactor_1_63  6
#define SDD_RingConfig_RingCrestFactor_1_73  7
#endif /* S220 */

/** The  SDD_MwiConfig  messages  are  used  to  write  and  read  the
    Message  Waiting  Indication  configuration parameters. */
typedef struct SDD_MwiConfig
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Message Waiting Voltage */
   IFX_uint32_t MwiVoltage : 16;
   /** Message Waiting Hook Threshold */
   IFX_uint32_t MwiThresh : 16;
   /** Message Waiting Slope */
   IFX_uint32_t MwiSlope : 16;
   /** Message Waiting On-time (Lamp ON) */
   IFX_uint32_t MwiOnTime : 8;
   /** Message Waiting Off-time (Lamp OFF) */
   IFX_uint32_t MwiOffTime : 8;
#else
   CMD_HEAD_LE;
   /** Message Waiting Hook Threshold */
   IFX_uint32_t MwiThresh : 16;
   /** Message Waiting Voltage */
   IFX_uint32_t MwiVoltage : 16;
   /** Message Waiting Off-time (Lamp OFF) */
   IFX_uint32_t MwiOffTime : 8;
   /** Message Waiting On-time (Lamp ON) */
   IFX_uint32_t MwiOnTime : 8;
   /** Message Waiting Slope */
   IFX_uint32_t MwiSlope : 16;
#endif
} __PACKED__ SDD_MwiConfig_t;
#define SDD_MwiConfig_ECMD    6
#define SDD_MwiConfig_LENGTH  8

/** The SDD_DcDcConfig  messages  are  used  to  write  and  read  the  DC/DC
    configuration  parameters,  which overwrite the default DC/DC settings
    in the SMART Device Driver. */
typedef struct SDD_DcDcConfig
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   #ifdef S220
   /** DCDC Offtime Coefficient */
   IFX_uint32_t DcDcOffC : 4;
   /** DC/DC Capacitance */
   IFX_uint32_t DcDcCap : 4;
   /** DC/DC Hardware Option */
   IFX_uint32_t DcDcHw : 4;
   /** Behaviour of VN during Ringing */
   IFX_uint32_t RingVN : 1;
   /** DC/DC Switch Output Enable */
   IFX_uint32_t DcDcSwEn : 1;
   /** Combined Low Power Mode */
   IFX_uint32_t CombLP : 1;
   /** DC/DC Switch Output Inverted */
   IFX_uint32_t DcDcSwInv : 1;
   /** Power Value for Boost Calculation */
   IFX_uint32_t P1 : 16;
   /** DC/DC Maximum On Time of Switching Transistor */
   IFX_uint32_t MaxOnTime : 8;
   /** Maximum Frequency of DC/DC Converter */
   IFX_uint32_t MaxFreq : 8;
   /** DC/DC On Time of Switching Transistor in STANDBY Mode */
   IFX_uint32_t OnTimeStby : 8;
   /** Maximum Frequency of DC/DC Converter in STANDBY Mode */
   IFX_uint32_t MaxFreqStby : 8;
   #else /* S220 */
   /** Reserved */
   IFX_uint32_t Res01 : 8;
   /** DC/DC Hardware Option */
   IFX_uint32_t DcDcHw : 4;
   /**  DC/DC Output Voltage Determined by Vcm DAC */
   IFX_uint32_t DcDcVconst : 1;
   /** DC/DC Switch Output Enable */
   IFX_uint32_t DcDcSwEn : 1;
   /** DC/DC Clock Every Second Cycle Low */
   IFX_uint32_t DcDcClkLow : 1;
   /** DC/DC Switch Output Inverted */
   IFX_uint32_t DcDcSwInv : 1;
   /** Reserved */
   IFX_uint32_t Res02 : 5;
   /** DC/DC Saw Tooth Generator Slope */
   IFX_uint32_t DcDcSaw : 3;
   /** DC/DC Frequency Counter */
   IFX_uint32_t DcDcFreq : 8;
   /** DC/DC Duty Cycle Counter for Idle Low Power */
   IFX_uint32_t DcDcDutyLp : 8;
   /** DC/DC Duty Cycle Counter */
   IFX_uint32_t DcDcDuty : 8;
   /**  DC/DC Output Voltage (When DcDcVconst = 1) */
   IFX_uint32_t DcDcVoltage : 16;
   #endif
#else
   CMD_HEAD_LE;
   #ifdef S220
   /** Power Value for Boost Calculation */
   IFX_uint32_t P1 : 16;
   /** DC/DC Switch Output Inverted */
   IFX_uint32_t DcDcSwInv : 1;
   /** Combined Low Power Mode */
   IFX_uint32_t CombLP : 1;
   /** DC/DC Switch Output Enable */
   IFX_uint32_t DcDcSwEn : 1;
   /** Behaviour of VN during Ringing */
   IFX_uint32_t RingVN : 1;
   /** DC/DC Hardware Option */
   IFX_uint32_t DcDcHw : 4;
   /** DC/DC Capacitance */
   IFX_uint32_t DcDcCap : 4;
   /** DCDC Offtime Coefficient */
   IFX_uint32_t DcDcOffC : 4;
   /** Maximum Frequency of DC/DC Converter in STANDBY Mode */
   IFX_uint32_t MaxFreqStby : 8;
   /** DC/DC On Time of Switching Transistor in STANDBY Mode */
   IFX_uint32_t OnTimeStby : 8;
   /** Maximum Frequency of DC/DC Converter */
   IFX_uint32_t MaxFreq : 8;
   /** DC/DC Maximum On Time of Switching Transistor */
   IFX_uint32_t MaxOnTime : 8;
   #else /* S220 */
   /** DC/DC Frequency Counter */
   IFX_uint32_t DcDcFreq : 8;
   /** DC/DC Saw Tooth Generator Slope */
   IFX_uint32_t DcDcSaw : 3;
   /** Reserved */
   IFX_uint32_t Res02 : 5;
   /** DC/DC Switch Output Inverted */
   IFX_uint32_t DcDcSwInv : 1;
   /** DC/DC Clock Every Second Cycle Low */
   IFX_uint32_t DcDcClkLow : 1;
   /** DC/DC Switch Output Enable */
   IFX_uint32_t DcDcSwEn : 1;
   /**  DC/DC Output Voltage Determined by Vcm DAC */
   IFX_uint32_t DcDcVconst : 1;
   /** DC/DC Hardware Option */
   IFX_uint32_t DcDcHw : 4;
   /** Reserved */
   IFX_uint32_t Res01 : 8;
   /**  DC/DC Output Voltage (When DcDcVconst = 1) */
   IFX_uint32_t DcDcVoltage : 16;
   /** DC/DC Duty Cycle Counter */
   IFX_uint32_t DcDcDuty : 8;
   /** DC/DC Duty Cycle Counter for Idle Low Power */
   IFX_uint32_t DcDcDutyLp : 8;
   #endif /* S220 */
#endif
} __PACKED__ SDD_DcDcConfig_t;
#define SDD_DcDcConfig_ECMD     0xC
#define SDD_DcDcConfig_LENGTH   8
/* Mask to check for combined or dedicated DC/DC Hardware setting. */
#define SDD_DcDcConfig_DcDcHw_Mask_COMBINED 0x1


/** The SDD_TxRxGain message sets the relative TX and RX levels. */
typedef struct SDD_TxRxGain
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Tx Relative Level Setting */
   IFX_uint32_t TxGain : 16;
   /** Rx Relative Level setting */
   IFX_uint32_t RxGain : 16;
#else
   CMD_HEAD_LE;
   /** Rx Relative Level setting */
   IFX_uint32_t RxGain : 16;
   /** Tx Relative Level Setting */
   IFX_uint32_t TxGain : 16;
#endif
} __PACKED__ SDD_TxRxGain_t;
#define SDD_TxRxGain_ECMD      0xE
#define SDD_TxRxGain_LENGTH    4

/** The SDD_Coeff message writes or reads programmable coefficients. */
typedef struct SDD_Coeff
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   #ifdef S220
      /** Destination */
      IFX_uint16_t Dest : 1;
      /** Offset of Coefficient Block */
      IFX_uint16_t BlockOffset : 10;
      /** Length of Coefficient Block */
      IFX_uint16_t BlockLength : 5;
      /** Coefficients or coefficients blocks */
      IFX_uint16_t Coefficients[27];
   #else /* S220 */
      /** Offset of Coefficient Block */
      IFX_uint16_t BlockOffset : 11;
      /** Length of Coefficient Block */
      IFX_uint16_t BlockLength : 5;
      /** Coefficients or coefficients blocks */
      IFX_uint16_t Coefficients[27];
   #endif /* S220 */
#else
   CMD_HEAD_LE;
   #ifdef S220
      /** Length of Coefficient Block */
      IFX_uint16_t BlockLength : 5;
      /** Offset of Coefficient Block */
      IFX_uint16_t BlockOffset : 10;
      /** Destination */
      IFX_uint16_t Dest : 1;
      /** Coefficients or coefficients blocks */
      IFX_uint16_t Coefficients[27];
   #else /* S220 */
      /** Length of Coefficient Block */
      IFX_uint16_t BlockLength : 5;
      /** Offset of Coefficient Block */
      IFX_uint16_t BlockOffset : 11;
      /** Coefficients or coefficients blocks */
      IFX_uint16_t Coefficients[27];
   #endif /* S220 */
#endif
} __PACKED__ SDD_Coeff_t;

#define SDD_Coeff_ECMD 0xD
#define SDD_Coeff_Dest_ASDSP  0
#define SDD_Coeff_Dest_ACDC   1
#define SDD_Coeff_MAXLENGTH   56


/** The  SDD_Opmode  message  is  used  to  write  and  read  the  parameters
    that  are  required  to  set  the  desired operating mode of the
    Analog Line. */
typedef struct SDD_Opmode
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   #ifdef S220
   /** Reserved */
   IFX_uint32_t Res02 : 8;
   /** Select Operating Mode */
   IFX_uint32_t OperatingMode : 8;
   #else /* S220 */
   /** Polarity */
   IFX_uint32_t Polarity : 1;
   /** Howler Tone Configuration */
   IFX_uint32_t Howler : 1;
   /** Start a TTX Burst / TTX Burst Running */
   IFX_uint32_t TtxBurst : 1;
   /** Select Party */
   IFX_uint32_t SelParty : 1;
   /** Light the Message Waiting Lamp */
   IFX_uint32_t MwiLamp : 1;
   /** Enable 5 kohm Resistor */
   IFX_uint32_t Res5kEn : 1;
   /** Set TIP wire to high impedance state */
   IFX_uint32_t HighImpTip : 1;
   /** Set RING wire to high impedance state */
   IFX_uint32_t HighImpRing : 1;
   /** Current Operating Mode (Read Only) */
   IFX_uint32_t CurOpMode : 4;
   /** Select Operating Mode */
   IFX_uint32_t OperatingMode : 4;
   #endif /* S220 */
   /** Reserved */
   IFX_uint32_t Res01 : 16;
#else
   CMD_HEAD_LE;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   #ifdef S220
   /** Select Operating Mode */
   IFX_uint32_t OperatingMode : 8;
   /** Reserved */
   IFX_uint32_t Res02 : 8;
   #else /* S220 */
   /** Select Operating Mode */
   IFX_uint32_t OperatingMode : 4;
   /** Current Operating Mode (Read Only) */
   IFX_uint32_t CurOpMode : 4;
   /** Set RING wire to high impedance state */
   IFX_uint32_t HighImpRing : 1;
   /** Set TIP wire to high impedance state */
   IFX_uint32_t HighImpTip : 1;
   /** Enable 5 kohm Resistor */
   IFX_uint32_t Res5kEn : 1;
   /** Light the Message Waiting Lamp */
   IFX_uint32_t MwiLamp : 1;
   /** Select Party */
   IFX_uint32_t SelParty : 1;
   /** Start a TTX Burst / TTX Burst Running */
   IFX_uint32_t TtxBurst : 1;
   /** Howler Tone Configuration */
   IFX_uint32_t Howler : 1;
   /** Polarity */
   IFX_uint32_t Polarity : 1;
   #endif /* S220 */
#endif
} __PACKED__ SDD_Opmode_t;
#define SDD_Opmode_ECMD        7
#define SDD_Opmode_LENGTH      4
#define SDD_Opmode_Disabled    0
#define SDD_Opmode_Ringing     1
#define SDD_Opmode_Standby     2
#define SDD_Opmode_Active      3
#define SDD_Opmode_Calibrate   6
#define SDD_Opmode_GR909       7
#define SDD_Opmode_Init        8
#ifdef S220
#define SDD_Opmode_Ground_Start        5
#define SDD_Opmode_ESD                11
#define SDD_Opmode_Active_MWL         12
#define SDD_Opmode_Active_HOWL        13
#define SDD_Opmode_Ringing_REVPOL     17
#define SDD_Opmode_Active_REVPOL      19
#define SDD_Opmode_Ground_Start_T2G   21
#define SDD_Opmode_PDF                22
#define SDD_Opmode_CAP_MEAS           23
#define SDD_Opmode_Active_HOWL_REVPOL 29
#else /* S220 */
#define SDD_Opmode_FXO         4
#define SDD_Opmode_Pol_Normal  0
#define SDD_Opmode_Pol_Reverse 1
#define SDD_Opmode_HT_Speech   0
#define SDD_Opmode_HT_Howler   1
#define SDD_Opmode_TTX_NO      0
#define SDD_Opmode_TTX_BURST   1
#define SDD_Opmode_MWL_Off     0
#define SDD_Opmode_MWL_On      1
#endif /* S220 */

/** The SDD_GR909Config messages are used to write and read the parameters
    used for the embedded linetesting mode of the SMART Device Driver. */
typedef struct SDD_GR909Config
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   #ifdef S220
   /** Reserved */
   IFX_uint32_t Res01 : 11;
   /** Settle Time after Discharge */
   IFX_uint32_t SettleTime : 5;
   #else /* S220 */
   /** Reserved */
   IFX_uint32_t Res01 : 6;
   /** Phone Detection Feature */
   IFX_uint32_t Pdf : 1;
   /** Reserved */
   IFX_uint32_t Res02 : 4;
   /** Hazardous Potential Test */
   IFX_uint32_t Hpt : 1;
   /** Foreign ElectroMotive Forces Test */
   IFX_uint32_t Femf : 1;
   /** Resistive Faults Test */
   IFX_uint32_t Rft : 1;
   /** Receiver Off-Hook Test */
   IFX_uint32_t Roh : 1;
   /** Ringer Impedance Test */
   IFX_uint32_t Rit : 1;
   #endif /* S220 */
   /** HPT Wire to GND AC Limit */
   IFX_uint32_t HptW2gAcLim : 16;
   /** HPT Wire to Wire AC Limit */
   IFX_uint32_t HptW2wAcLim : 16;
   /** HPT Wire to GND DC Limit */
   IFX_uint32_t HptW2gDcLim : 16;
   /** HPT Wire to Wire DC Limit */
   IFX_uint32_t HptW2wDcLim : 16;
   /** FEMF Wire to GND AC Limit */
   IFX_uint32_t FemfW2gAcLim : 16;
   /** FEMF Wire to Wire AC Limit */
   IFX_uint32_t FemfW2wAcLim : 16;
   /** FEMF Wire to GND DC Limit */
   IFX_uint32_t FemfW2gDcLim : 16;
   /** FEMF Wire to Wire DC Limit */
   IFX_uint32_t FemfW2wDcLim : 16;
   /** RFT Resistance Limit */
   IFX_uint32_t RftResLim : 16;
   /** ROH Linearity Limit */
   IFX_uint32_t RohLinLim : 16;
   /** RIT Lower Limit */
   IFX_uint32_t RitLowLim : 16;
   /** RIT Higher Limit */
   IFX_uint32_t RitHighLim : 16;
   /** Reserved */
   IFX_uint32_t Res03 : 16;
#else
   CMD_HEAD_LE;
   /** HPT Wire to GND AC Limit */
   IFX_uint32_t HptW2gAcLim : 16;
   #ifdef S220
   /** Settle Time after Discharge */
   IFX_uint32_t SettleTime : 5;
   /** Reserved */
   IFX_uint32_t Res01 : 11;
   #else /* S220 */
   /** Ringer Impedance Test */
   IFX_uint32_t Rit : 1;
   /** Receiver Off-Hook Test */
   IFX_uint32_t Roh : 1;
   /** Resistive Faults Test */
   IFX_uint32_t Rft : 1;
   /** Foreign ElectroMotive Forces Test */
   IFX_uint32_t Femf : 1;
   /** Hazardous Potential Test */
   IFX_uint32_t Hpt : 1;
   /** Reserved */
   IFX_uint32_t Res02 : 4;
   /** Phone Detection Feature */
   IFX_uint32_t Pdf : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 6;
   #endif /* S220 */
   /** HPT Wire to GND DC Limit */
   IFX_uint32_t HptW2gDcLim : 16;
   /** HPT Wire to Wire AC Limit */
   IFX_uint32_t HptW2wAcLim : 16;
   /** FEMF Wire to GND AC Limit */
   IFX_uint32_t FemfW2gAcLim : 16;
   /** HPT Wire to Wire DC Limit */
   IFX_uint32_t HptW2wDcLim : 16;
   /** FEMF Wire to GND DC Limit */
   IFX_uint32_t FemfW2gDcLim : 16;
   /** FEMF Wire to Wire AC Limit */
   IFX_uint32_t FemfW2wAcLim : 16;
   /** RFT Resistance Limit */
   IFX_uint32_t RftResLim : 16;
   /** FEMF Wire to Wire DC Limit */
   IFX_uint32_t FemfW2wDcLim : 16;
   /** RIT Lower Limit */
   IFX_uint32_t RitLowLim : 16;
   /** ROH Linearity Limit */
   IFX_uint32_t RohLinLim : 16;
   /** Reserved */
   IFX_uint32_t Res03 : 16;
   /** RIT Higher Limit */
   IFX_uint32_t RitHighLim : 16;
#endif
} __PACKED__ SDD_GR909Config_t;
#define SDD_GR909Config_ECMD   8
#define SDD_GR909Config_LENGTH 0x1C

#ifdef S220
/** The SDD_OpenLoopCalibration message is used to configure the open
    loop conductances for each channel. */
typedef struct SDD_OpenLoopCalibration
{
   #if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Open Loop Conductance Ring To Ground */
   IFX_uint32_t GRG : 8;
   /** Open Loop Conductance Tip To Ground */
   IFX_uint32_t GTG : 8;
   /** Open Loop Conductance Tip To Ring */
   IFX_uint32_t GTR : 8;
   /** Reserved */
   IFX_uint32_t Res01 : 8;
   #else
   CMD_HEAD_LE;
   /** Reserved */
   IFX_uint32_t Res01 : 8;
   /** Open Loop Conductance Tip To Ring */
   IFX_uint32_t GTR : 8;
   /** Open Loop Conductance Tip To Ground */
   IFX_uint32_t GTG : 8;
   /** Open Loop Conductance Ring To Ground */
   IFX_uint32_t GRG : 8;
   #endif
} __PACKED__ SDD_OpenLoopCalibration_t;
#define SDD_OpenLoopCalibration_ECMD   11
#define SDD_OpenLoopCalibration_LENGTH 4

/** The SDD_AcLevelMeterControl message starts and stops the AC level meter
    measurement.
*/
typedef struct SDD_AcLevelMeterControl
{
   #if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Enable */
   IFX_uint32_t EN : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 31;
   #else
   CMD_HEAD_LE;
   /** Reserved */
   IFX_uint32_t Res01 : 31;
   /** Enable */
   IFX_uint32_t EN : 1;
   #endif
} __PACKED__ SDD_AcLevelMeterControl_t;
#define SDD_AcLevelMeterControl_ECMD    17
#define SDD_AcLevelMeterControl_LENGTH  4
#define SDD_AcLevelMeterControl_ENABLE  1
#define SDD_AcLevelMeterControl_DISABLE 0

/** The SDD_AcLevelMeterConfig message configures the AC level meter. */
typedef struct SDD_AcLevelMeterConfig
{
   #if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Bandwidth Coefficient */
   IFX_uint32_t BW : 16;
   /** Center Frequency Coefficient */
   IFX_uint32_t CF : 16;
   /** Reserved */
   IFX_uint32_t Res01 : 14;
   /** Bandpass enable */
   IFX_uint32_t BP : 1;
   /** Transhybrid enable */
   IFX_uint32_t TH : 1;
   /** AC Delay */
   IFX_uint32_t Del : 7;
   /** AC Integration Time */
   IFX_uint32_t Int : 9;
   #else
   CMD_HEAD_LE;
   /** Center Frequency Coefficient */
   IFX_uint32_t CF : 16;
   /** Bandwidth Coefficient */
   IFX_uint32_t BW : 16;
   /** AC Integration Time */
   IFX_uint32_t Int : 9;
   /** AC Delay */
   IFX_uint32_t Del : 7;
   /** Transhybrid enable */
   IFX_uint32_t TH : 1;
   /** Bandpass enable */
   IFX_uint32_t BP : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 14;
   #endif
} __PACKED__ SDD_AcLevelMeterConfig_t;
#define SDD_AcLevelMeterConfig_ECMD    18
#define SDD_AcLevelMeterConfig_LENGTH  8
#endif /* S220 */

/** The SDD_GR909ResultsRead message is used to read the results of the
    embedded linetesting of the SMART Device Driver. */
typedef struct SDD_GR909ResultsRead
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** HPT Test Valid */
   IFX_uint32_t HptValid : 1;
   /** FEMF Test Valid */
   IFX_uint32_t FemfValid : 1;
   /** RFT Test Valid */
   IFX_uint32_t RftValid : 1;
   /** ROH Test Valid */
   IFX_uint32_t RohValid : 1;
   /** RIT Test Valid */
   IFX_uint32_t RitValid : 1;
   #ifdef S220
   /** Reserved */
   IFX_uint32_t Res01 : 11;
   #else /* S220 */
   /** Reserved */
   IFX_uint32_t Res01 : 6;
   /** GR 909 Busy */
   IFX_uint32_t Busy : 1;
   /** Reserved */
   IFX_uint32_t Res02 : 4;
   #endif /* S220 */
   /** HPT Test Passed */
   IFX_uint32_t HptPass : 1;
   /** FEMF Test Passed */
   IFX_uint32_t FemfPass : 1;
   /** RFT Test Passed */
   IFX_uint32_t RftPass : 1;
   /** ROH Test Passed */
   IFX_uint32_t RohPass : 1;
   /** RIT Test Passed */
   IFX_uint32_t RitPass : 1;
   /** Reserved */
   IFX_uint32_t Res03 : 11;
   /** Test Result HPT or FEMF AC Ring to GND */
   IFX_uint32_t HptAcR2g : 16;
   /** Test Result HPT or FEMF AC Tip to GND */
   IFX_uint32_t HptAcT2g : 16;
   /** Test Result HPT or FEMF AC Tip to Ring */
   IFX_uint32_t HptAcT2r : 16;
   /** Test Result HPT or FEMF DC Ring to GND */
   IFX_uint32_t HptDcR2g : 16;
   /** Test Result HPT or FEMF DC Tip to GND */
   IFX_uint32_t HptDcT2g : 16;
   /** Test Result HPT or FEMF DC Tip to Ring */
   IFX_uint32_t HptDcT2r : 16;
   /** Test Result RFT Ring to GND */
   IFX_uint32_t RftR2g : 16;
   /** Test Result RFT Tip to GND */
   IFX_uint32_t RftT2g : 16;
   /** Test Result RFT Tip to Ring */
   IFX_uint32_t RftT2r : 16;
   /** Test Result ROH Low */
   IFX_uint32_t RohLow : 16;
   /** Test Result ROH High */
   IFX_uint32_t RohHigh : 16;
   /** Test Result RIT */
   IFX_uint32_t RitRes : 16;
#else
   CMD_HEAD_LE;
   /** Reserved */
   IFX_uint32_t Res03 : 11;
   /** RIT Test Passed */
   IFX_uint32_t RitPass : 1;
   /** ROH Test Passed */
   IFX_uint32_t RohPass : 1;
   /** RFT Test Passed */
   IFX_uint32_t RftPass : 1;
   /** FEMF Test Passed */
   IFX_uint32_t FemfPass : 1;
   /** HPT Test Passed */
   IFX_uint32_t HptPass : 1;
   #ifdef S220
   /** Reserved */
   IFX_uint32_t Res01 : 11;
   #else /* S220 */
   /** Reserved */
   IFX_uint32_t Res02 : 4;
   /** GR 909 Busy */
   IFX_uint32_t Busy : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 6;
   #endif /* S220 */
   /** RIT Test Valid */
   IFX_uint32_t RitValid : 1;
   /** ROH Test Valid */
   IFX_uint32_t RohValid : 1;
   /** RFT Test Valid */
   IFX_uint32_t RftValid : 1;
   /** FEMF Test Valid */
   IFX_uint32_t FemfValid : 1;
   /** HPT Test Valid */
   IFX_uint32_t HptValid : 1;
   /** Test Result HPT or FEMF AC Tip to GND */
   IFX_uint32_t HptAcT2g : 16;
   /** Test Result HPT or FEMF AC Ring to GND */
   IFX_uint32_t HptAcR2g : 16;
   /** Test Result HPT or FEMF DC Ring to GND */
   IFX_uint32_t HptDcR2g : 16;
   /** Test Result HPT or FEMF AC Tip to Ring */
   IFX_uint32_t HptAcT2r : 16;
   /** Test Result HPT or FEMF DC Tip to Ring */
   IFX_uint32_t HptDcT2r : 16;
   /** Test Result HPT or FEMF DC Tip to GND */
   IFX_uint32_t HptDcT2g : 16;
   /** Test Result RFT Tip to GND */
   IFX_uint32_t RftT2g : 16;
   /** Test Result RFT Ring to GND */
   IFX_uint32_t RftR2g : 16;
   /** Test Result ROH Low */
   IFX_uint32_t RohLow : 16;
   /** Test Result RFT Tip to Ring */
   IFX_uint32_t RftT2r : 16;
   /** Test Result RIT */
   IFX_uint32_t RitRes : 16;
   /** Test Result ROH High */
   IFX_uint32_t RohHigh : 16;
#endif
} __PACKED__ SDD_GR909ResultsRead_t;
#define SDD_GR909ResultsRead_ECMD    9
#define SDD_GR909ResultsRead_PF_LENGTH                      0x04
#define SDD_GR909ResultsRead_PF_HPT_LENGTH                  0x10
#define SDD_GR909ResultsRead_PF_HPT_RFT_LENGTH              0x18
#define SDD_GR909ResultsRead_PF_HPT_RFT_ROH_LENGTH          0x1C
#define SDD_GR909ResultsRead_PF_HPT_RFT_ROH_RIT_LENGTH      0x1C
#define SDD_GR909ResultsRead_MAX_LENGTH                     0x1C

/** The  SDD_GR909PhoneDetection messages  is  used  to  read  the  measured
    capacitance  value  of  a  phone  in onhook state. */
typedef struct SDD_GR909PhoneDetection
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Capacitance of the phone */
   IFX_uint32_t Capacitance : 32;
#else
   CMD_HEAD_LE;
   /** Capacitance of the phone */
   IFX_uint32_t Capacitance : 32;
#endif
} __PACKED__ SDD_GR909PhoneDetection_t;
#define SDD_GR909PhoneDetection_ECMD    0
#define SDD_GR909PhoneDetection_LENGTH  4

#ifdef S220
/** The SDD_CapacitanceMeasurementRead message is used to read the capacitance
    values from ring to ground, tip to ground and tip to ring. It can also be
    used for phone detection. */
typedef struct SDD_CapacitanceMeasurementRead
{
   #if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Capacitance Ring to Ground */
   IFX_uint32_t CapR2G : 16;
    /** Capacitance Tip to Ground */
   IFX_uint32_t CapT2G : 16;
   /** Capacitance Tip to Ring */
   IFX_uint32_t CapT2R : 16;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   #else
   CMD_HEAD_LE;
   /** Capacitance Tip to Ground */
   IFX_uint32_t CapT2G : 16;
   /** Capacitance Ring to Ground */
   IFX_uint32_t CapR2G : 16;
   /** Reserved */
   IFX_uint32_t Res01 : 16;
   /** Capacitance Tip to Ring */
   IFX_uint32_t CapT2R : 16;
   #endif
} __PACKED__ SDD_CapacitanceMeasurementRead_t;
#define SDD_CapacitanceMeasurementRead_ECMD    16
#define SDD_CapacitanceMeasurementRead_LENGTH  8

/** The SDD_AcLevelMeterResult message configures the AC level meter. */
typedef struct SDD_AcLevelMeterResult
{
    #if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
    CMD_HEAD_BE;
    /** AC Levelmeter Inband Result */
    IFX_uint32_t AcInb : 24;
    /** AC Levelmeter Inband Result Shift */
    IFX_uint32_t AcInbSh : 8;
    /** AC Levelmeter Outband Result */
    IFX_uint32_t AcOutb : 24;
    /** AC Levelmeter Outband Result Shift */
    IFX_uint32_t AcOutbSh : 8;
    #else
    CMD_HEAD_LE;
    /** AC Levelmeter Inband Result Shift */
    IFX_uint32_t AcInbSh : 8;
    /** AC Levelmeter Inband Result */
    IFX_uint32_t AcInb : 24;
    /** AC Levelmeter Outband Result Shift */
    IFX_uint32_t AcOutbSh : 8;
    /** AC Levelmeter Outband Result */
    IFX_uint32_t AcOutb : 24;
   #endif
} __PACKED__ SDD_AcLevelMeterResult_t;
#define SDD_AcLevelMeterResult_ECMD    19
#define SDD_AcLevelMeterResult_LENGTH   8
#endif /* S220 */

/** The SDD_En8kLoop message is used to read all the debugging information
    and set the debug control bits of the SMART Device Driver */
typedef struct SDD_En8kLoop
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Reserved */
   IFX_uint32_t Res01 : 31;
   /** Enable 8k Test Loop in ASDSP */
   IFX_uint32_t En8kLoop : 1;
#else
   CMD_HEAD_LE;
   /** Enable 8k Test Loop in ASDSP */
   IFX_uint32_t En8kLoop : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 31;
#endif
} __PACKED__ SDD_En8kLoop_t;
#define SDD_En8kLoop_ECMD      0x1E
#define SDD_En8kLoop_LENGTH    4
#define SDD_En8kLoop_DISABLE  0
#define SDD_En8kLoop_ENABLE   1

/** The SDD_DartDebug message is used to directly read or write the HW
    registers on the ARC bus or the DART-IO bus.*/
typedef struct SDD_DartDebug
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Write/Read Debug Data */
   IFX_uint32_t RWD : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 3;
   /** Debug Error Code */
   IFX_uint32_t DebugError : 5;
   /** Reserved */
   IFX_uint32_t Res02 : 18;
   /** GPIO debug mode */
   IFX_uint32_t GPIODebug : 2;
   /** Debug Bus Type */
   IFX_uint32_t DebugType : 4;
   /** Debug Bus or Register Address */
   IFX_uint32_t DartDebugAddr :32;
   /** Debug Data */
   IFX_uint32_t DartDebugData : 32;
#else
   CMD_HEAD_LE;
   /** Debug Bus Type */
   IFX_uint32_t DebugType : 4;
   /** GPIO debug mode */
   IFX_uint32_t GPIODebug : 2;
   /** Reserved */
   IFX_uint32_t Res02 : 18;
   /** Debug Error Code */
   IFX_uint32_t DebugError : 5;
   /** Reserved */
   IFX_uint32_t Res01 : 3;
   /** Write/Read Debug Data */
   IFX_uint32_t RWD : 1;
   /** Debug Bus or Register Address */
   IFX_uint32_t DartDebugAddr :32;
   /** Debug Data */
   IFX_uint32_t DartDebugData : 32;
#endif
} __PACKED__ SDD_DartDebug_t;
#define SDD_DartDebug_ECMD    0x1F
#define SDD_DartDebug_LENGTH  0x0C
#define SDD_DartDebug_WRITE_DATA   0
#define SDD_DartDebug_READ_DATA    1
#define SDD_DartDebug_OFF     0x00
#define SDD_DartDebug_HOOK_TF 0x01
#define SDD_DartDebug_GK_GF   0x02
#define SDD_DartDebug_MIPS    0x00
#define SDD_DartDebug_DART    0x01
#define SDD_DartDebug_GPIO    0x08

/** This command configures and activates the PCM interface module and must
    be sent before a PCM channel can be configured. */
struct SDD_PCM_IF_CTRL
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
    /** Enable */
    IFX_uint32_t EN : 1;
    /** Reserved */
    IFX_uint32_t Res01 : 4;
    /** Transmit Bit Offset */
    IFX_uint32_t XOFF : 3;
    /** Double Bit Clock */
    IFX_uint32_t DBL : 1;
    /** Transmit Slope */
    IFX_uint32_t XS : 1;
    /** Receive Slope */
    IFX_uint32_t RS : 1;
    /** Bit 0 Drive Length */
    IFX_uint32_t DRV0 : 1;
    /** Shift Control */
    IFX_uint32_t SH : 1;
    /** Receive Bit Offset */
    IFX_uint32_t ROFF : 3;
    /** Reserved */
    IFX_uint32_t Res02 : 16;
#else
   CMD_HEAD_LE;
    /** Reserved */
    IFX_uint32_t Res02 : 16;
   /** Receive Bit Offset */
    IFX_uint32_t ROFF : 3;
    /** Shift Control */
    IFX_uint32_t SH : 1;
   /** Bit 0 Drive Length */
    IFX_uint32_t DRV0 : 1;
   /** Receive Slope */
    IFX_uint32_t RS : 1;
   /** Transmit Slope */
    IFX_uint32_t XS : 1;
   /** Double Bit Clock */
    IFX_uint32_t DBL : 1;
   /** Transmit Bit Offset */
    IFX_uint32_t XOFF : 3;
   /** Reserved */
    IFX_uint32_t Res01 : 4;
   /** Enable */
    IFX_uint32_t EN : 1;
#endif
} __PACKED__;

#define SDD_PCM_IF_CTRL_ECMD 0x00
#define SDD_PCM_IF_CTRL_LENGTH 0x04
#define SDD_PCM_IF_CTRL_ENABLE 0x1
#define SDD_PCM_IF_CTRL_DISABLE 0x0
#define SDD_PCM_IF_CTRL_DBL_SINGLE 0
#define SDD_PCM_IF_CTRL_DBL_DOUBLE 1
#define SDD_PCM_IF_CTRL_XS_RISING_EDGE 0
#define SDD_PCM_IF_CTRL_XS_FALLING_EDGE 1
#define SDD_PCM_IF_CTRL_RS_RISING_EDGE 0
#define SDD_PCM_IF_CTRL_RS_FALLING_EDGE 1
#define SDD_PCM_IF_CTRL_DRV0_ENTIRE_PERIOD 0
#define SDD_PCM_IF_CTRL_DRV0_FIRST_HALF 1
#define SDD_PCM_IF_CTRL_SH_NO_SHIFT 0
#define SDD_PCM_IF_CTRL_SH_SHIFT 1

/** This command configures a PCM channel. */
struct SDD_PCM_CH_CTRL
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
    /** Enable */
    IFX_uint32_t EN : 1;
    /** Coder */
    IFX_uint32_t COD : 3;
    /** Wideband 16kHz */
    IFX_uint32_t WIDE : 1;
    /** Wideband PCM Time Slot Configuration Bit */
    IFX_uint32_t WBTSC : 1;
   #ifdef S220
    /** Reserved */
    IFX_uint32_t Res01 : 11;
   #else /* S220 */
    /** Reserved */
    IFX_uint32_t Res01 : 10;
    /** Transmit Highway */
    IFX_uint32_t X_HW : 1;
   #endif /* S220 */
    /** Transmit Highway Time Slot */
    IFX_uint32_t XTS : 7;
   #ifdef S220
    /** Reserved */
    IFX_uint32_t Res02 : 1;
   #else /* S220 */
    /** Receive Highway */
    IFX_uint32_t R_HW : 1;
   #endif /* S220 */
    /** Receive Highway Time Slot */
    IFX_uint32_t RTS : 7;
#else
   CMD_HEAD_LE;
    /** Receive Highway Time Slot */
    IFX_uint32_t RTS : 7;
   #ifdef S220
    /** Reserved */
    IFX_uint32_t Res02 : 1;
   #else /* S220 */
   /** Receive Highway */
    IFX_uint32_t R_HW : 1;
   #endif /* S220 */
   /** Transmit Highway Time Slot */
    IFX_uint32_t XTS : 7;
   #ifdef S220
    /** Reserved */
    IFX_uint32_t Res01 : 11;
   #else /* S220 */
   /** Transmit Highway */
    IFX_uint32_t X_HW : 1;
   /** Reserved */
    IFX_uint32_t Res01 : 10;
   #endif /* S220 */
    /** Wideband PCM Time Slot Configuration Bit */
    IFX_uint32_t WBTSC : 1;
   /** Wideband 16kHz */
    IFX_uint32_t WIDE : 1;
   /** Coder */
    IFX_uint32_t COD : 3;
   /** Enable */
    IFX_uint32_t EN : 1;
#endif
} __PACKED__;

#define SDD_PCM_CH_CTRL_ECMD 0x01
#define SDD_PCM_CH_CTRL_LENGTH 0x04
#define SDD_PCM_CH_CTRL_ENABLE 0x1
#define SDD_PCM_CH_CTRL_DISABLE 0x0
#define SDD_PCM_CH_CTRL_COD_LINEAR_16BIT 0x0
#define SDD_PCM_CH_CTRL_COD_ALAW_8BIT 0x2
#define SDD_PCM_CH_CTRL_COD_ULAW_8BIT 0x3
#define SDD_PCM_CH_CTRL_WIDE_8KHZ 0x0
#define SDD_PCM_CH_CTRL_WIDE_16KHZ 0x1
#define SDD_PCM_CH_CTRL_WBTSC_CONSECUTIVE 0x0
#define SDD_PCM_CH_CTRL_WBTSC_SPLIT 0x1


/** SDD_FxoHookSwitch */
typedef struct SDD_FXO_HOOK_SWITCH
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** HOOK switch */
   IFX_uint32_t HF : 2;
   /** Reserved */
   IFX_uint32_t Res01 : 30;
#else
   CMD_HEAD_LE;
   /** Reserved */
   IFX_uint32_t Res01 : 30;
   /** HOOK switch */
   IFX_uint32_t HF : 2;
#endif
} __PACKED__ SDD_FxoHookSwitch_t;

#define SDD_FxoHookSwitch_ECMD   0x1B
#define SDD_FxoHookSwitch_LEN       4
#define SDD_FxoHookSwitch_HF_ON     0
#define SDD_FxoHookSwitch_HF_OFF    1
#define SDD_FxoHookSwitch_HF_FLASH  2


/** SDD_FxoConfig */
typedef struct SDD_FXO_CONFIG
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   CMD_HEAD_BE;
   /** Reserved */
   IFX_uint32_t Res01 : 3;
   /** Timeout for transition to low power idle state */
   IFX_uint32_t TACT : 5;
   /** Debounce timer 2 */
   IFX_uint32_t DEB2 : 8;
   /** Maximum detection time for an open switching interval */
   IFX_uint32_t OSIT : 8;
   /** Flash time */
   IFX_uint32_t FLASHT : 8;
   /** Debounce timer 1 */
   IFX_uint32_t DEB1 : 8;
   /** Reserved */
   IFX_uint32_t Res02 : 24;
#else
   CMD_HEAD_LE;
   /** Flash time */
   IFX_uint32_t FLASHT : 8;
   /** Maximum detection time for an open switching interval */
   IFX_uint32_t OSIT : 8;
   /** Debounce timer 2 */
   IFX_uint32_t DEB2 : 8;
   /** Timeout for transition to low power idle state */
   IFX_uint32_t TACT : 5;
   /** Reserved */
   IFX_uint32_t Res01 : 3;
   /** Reserved */
   IFX_uint32_t Res02 : 24;
   /** Debounce timer 1 */
   IFX_uint32_t DEB1 : 8;
#endif
} __PACKED__ SDD_FxoConfig_t;

#define SDD_FxoConfig_ECMD  0x1A
#define SDD_FxoConfig_LENGTH   8

/* ============================= */
/* Global Variables              */
/* ============================= */

/* ============================= */
/* Global function declaration   */
/* ============================= */

#endif /* _drv_sdd_fw_cmd_H */
