#ifndef _drv_sdd_int_evt_H
#define _drv_sdd_int_evt_H
/******************************************************************************

                            Copyright (c) 2013-2015
                        Lantiq Beteiligungs-GmbH & Co.KG
                             http://www.lantiq.com

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

/**
   \file drv_sdd_int_evt.h
   This file contains the implementation of the interrupt handler.

   \remarks
*/

/* ========================================================================== */
/*                             Macro definitions                              */
/* ========================================================================== */
#ifndef __PACKED__
   #if defined (__GNUC__) || defined (__GNUG__)
      /* GNU C or C++ compiler */
      #define __PACKED__ __attribute__ ((packed))
   #elif !defined (__PACKED__)
      #define __PACKED__      /* nothing */
   #endif
#endif

#define EVENT_MAX_WORDS 3

/* ----- Common Event Header ----- */
/* Command Type for events */
#define EVT_CMD_EVT     9

/* Subcommand coding for events */
#define EVT_MOD_SYS     7
#define EVT_MOD_ALI     1

/* Length of the event header alone */
#define EVT_HDR_LEN     4

#define EVT_HEAD_BE           \
   /** Reserved */            \
   IFX_uint32_t ResHead00 : 3;\
   /** Command Type */        \
   IFX_uint32_t CMD : 5;      \
   /** Channel / Reserved */  \
   IFX_uint32_t CHAN : 8;     \
   /** Command Mode */        \
   IFX_uint32_t MOD : 3;      \
   /** Command Sub-Mode */    \
   IFX_uint32_t ECMD : 5;     \
   /** Length */              \
   IFX_uint32_t LENGTH : 8

#define EVT_HEAD_LE           \
   /** Length */              \
   IFX_uint32_t LENGTH : 8;   \
   /** Command Sub-Mode */    \
   IFX_uint32_t ECMD : 5;     \
   /** Command Mode */        \
   IFX_uint32_t MOD : 3;      \
   /** Channel / Reserved */  \
   IFX_uint32_t CHAN : 8;     \
   /** Command Type */        \
   IFX_uint32_t CMD : 5;      \
   /** Reserved */            \
   IFX_uint32_t ResHead00 : 3


/* ========================================================================== */
/*                             Type definitions                               */
/* ========================================================================== */
#ifdef S220
struct EVT_BOOT_FIN
{
   #if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   EVT_HEAD_BE;
   #else
   EVT_HEAD_LE;
   #endif
} __PACKED__;
#define EVT_BOOT_FIN_ECMD 0
#define EVT_BOOT_FIN_LENGTH 0
#endif /* S220 */

struct EVT_CERR
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   EVT_HEAD_BE;
   /** Reserved */
   IFX_uint32_t Res00 : 18;
   /** Error Cause */
   IFX_uint32_t CMDERR : 14;
   /** Command Header of Command */
   IFX_uint32_t CMDHDR : 32;
#else
   EVT_HEAD_LE;
   /** Error Cause */
   IFX_uint32_t CMDERR : 14;
   /** Reserved */
   IFX_uint32_t Res00 : 18;
   /** Command Header of Command */
   IFX_uint32_t CMDHDR : 32;
#endif
} __PACKED__;

#define EVT_CERR_ECMD 2
#define EVT_CERR_LENGTH 8
#ifdef S220
#define EVT_CERR_CMDERR_OK 0
#define EVT_CERR_CMDERR_BAD_CH 1
#define EVT_CERR_CMDERR_BAD_LEN 2
#define EVT_CERR_CMDERR_BAD_HEADER 3
#else /* S220 */
#define EVT_CERR_CMDERR_NO_ERR 0
#define EVT_CERR_CMDERR_BAD_CH 2
#define EVT_CERR_CMDERR_BAD_LEN 3
#define EVT_CERR_CMDERR_BAD_RW 13
#define EVT_CERR_CMDERR_BAD_CMD 35
#define EVT_CERR_CMDERR_BAD_MOD 36
#define EVT_CERR_CMDERR_BAD_ECMD 37
#endif /* S220 */

struct EVT_InternalError
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   EVT_HEAD_BE;
   #ifdef S220
   /** Reserved */
   IFX_uint32_t Res00 : 27;
   /** Event Code */
   IFX_uint32_t ERREVT : 5;
   #else /* S220 */
   /** Reserved */
   IFX_uint32_t Res00 : 24;
   /** Checksum Error */
   IFX_uint32_t CSE : 1;
   /** Internal Event-Box Overflow */
   IFX_uint32_t EBO : 1;
   /** Reserved */
   IFX_uint32_t Res01 : 6;
   #endif /* S220 */
#else
   EVT_HEAD_LE;
   #ifdef S220
   /** Event Code */
   IFX_uint32_t ERREVT : 5;
   /** Reserved */
   IFX_uint32_t Res00 : 27;
   #else /* S220 */
   /** Reserved */
   IFX_uint32_t Res01 : 6;
   /** Internal Event-Box Overflow */
   IFX_uint32_t EBO : 1;
   /** Checksum Error */
   IFX_uint32_t CSE : 1;
   /** Reserved */
   IFX_uint32_t Res00 : 24;
   #endif /* S220 */
#endif
} __PACKED__;

#define EVT_InternalError_ECMD   11
#define EVT_InternalError_LENGTH  4
#ifdef S220
#define EVT_InternalError_RES           0
#define EVT_InternalError_EQO           1
#define EVT_InternalError_CLK_FAIL      2
#define EVT_InternalError_SYNC_FAIL     3
#define EVT_InternalError_DCDCCFG_FAIL  4
#endif /* S220 */

struct EVT_SDD
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   EVT_HEAD_BE;
   /** Time Stamp */
   IFX_uint32_t TIMESTAMP : 16;
   #ifdef S220
   /** Analog line operating mode */
   IFX_uint32_t OPMODE : 8;
   #else /* S220 */
   /** Reserved */
   IFX_uint32_t Res01 : 4;
   /** Analog line operating mode */
   IFX_uint32_t OPMODE : 4;
   #endif /* S220 */
   /** Reserved */
   IFX_uint32_t Res02 : 3;
   /** Event code */
   IFX_uint32_t EVT : 5;
#else
   EVT_HEAD_LE;
   /** Event code */
   IFX_uint32_t EVT : 5;
   /** Reserved */
   IFX_uint32_t Res02 : 3;
   #ifdef S220
   /** Analog line operating mode */
   IFX_uint32_t OPMODE : 8;
   #else /* S220 */
   /** Analog line operating mode */
   IFX_uint32_t OPMODE : 4;
   /** Reserved */
   IFX_uint32_t Res01 : 4;
   /** Time Stamp */
   #endif /* S220 */
   IFX_uint32_t TIMESTAMP : 16;
#endif
} __PACKED__;

#define EVT_SDD_LENGTH 4
#define EVT_SDD_EVT_OTEMP         0
#define EVT_SDD_EVT_LTEST         1
#define EVT_SDD_EVT_GF            3
#define EVT_SDD_EVT_GK            4
#define EVT_SDD_EVT_OPC           5
#define EVT_SDD_EVT_COEFE         7
#define EVT_SDD_EVT_ONH           9
#define EVT_SDD_EVT_OFFH          10
#define EVT_SDD_EVT_GF_FIN        12
#define EVT_SDD_EVT_GK_FIN        13
#define EVT_SDD_EVT_OTEMP_FIN     14
#define EVT_SDD_EVT_SSI_CRASH     15
#define EVT_SDD_EVT_SSI_CRASH_FIN 16
#define EVT_SDD_EVT_OMI           17
#ifdef S220
#define EVT_SDD_ECMD 4
#define EVT_SDD_EVT_OPM_DIS       18
#define EVT_SDD_EVT_LT_ABORT      19
#define EVT_SDD_EVT_TTX_FIN       20
#define EVT_SDD_EVT_AC_LM_FIN     21
#define EVT_SDD_EVT_DATALINK_ERR  29
#define EVT_SDD_EVT_DCDCCFG_FAIL  30
#else /* S220 */
#define EVT_SDD_ECMD 0
#define EVT_SDD_EVT_CORCE         6
#define EVT_SDD_EVT_TTX_FIN       8
#define EVT_SDD_EVT_FXO_RING_ON   21
#define EVT_SDD_EVT_FXO_RING_OFF  22
#define EVT_SDD_EVT_FXO_BATT_ON   23
#define EVT_SDD_EVT_FXO_BATT_OFF  24
#define EVT_SDD_EVT_FXO_OSI_END   25
#define EVT_SDD_EVT_FXO_APOH_ON   26
#define EVT_SDD_EVT_FXO_APOH_OFF  27
#define EVT_SDD_EVT_FXO_POL_REV   28
#define EVT_SDD_EVT_FXO_POL_NOR   29
#endif /* S220 */

struct FW_Event_Header
{
#if IFXOS_BYTE_ORDER == IFXOS_BIG_ENDIAN
   EVT_HEAD_BE;
#else
   EVT_HEAD_LE;
#endif
} __PACKED__;


typedef union
{
   /* placeholder */
   IFX_uint32_t val[EVENT_MAX_WORDS];
   /* message header */
   struct FW_Event_Header hdr;
   /* Command Error Event */
   struct EVT_CERR cerr;
   /* Internal Error Event */
   struct EVT_InternalError int_err;
   /* SDD Event */
   struct EVT_SDD sdd;
} FW_Event_t;

#endif /* _drv_sdd_int_evt_H */
